import tgram
from typing import List
from typing import Union
from tgram.types import InlineKeyboardMarkup
from tgram.types import LabeledPrice
from tgram.types import Message
from tgram.types import ReplyParameters

from tgram.utils import convert_to_inline_keyboard_markup


class SendInvoice:
    async def send_invoice(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        title: str,
        description: str,
        payload: str,
        currency: str,
        prices: List[LabeledPrice],
        message_thread_id: int = None,
        provider_token: str = None,
        max_tip_amount: int = None,
        suggested_tip_amounts: List[int] = None,
        start_parameter: str = None,
        provider_data: str = None,
        photo_url: str = None,
        photo_size: int = None,
        photo_width: int = None,
        photo_height: int = None,
        need_name: bool = None,
        need_phone_number: bool = None,
        need_email: bool = None,
        need_shipping_address: bool = None,
        send_phone_number_to_provider: bool = None,
        send_email_to_provider: bool = None,
        is_flexible: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: InlineKeyboardMarkup = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        Sends invoice.

        Telegram documentation: https://core.telegram.org/bots/api#sendinvoice

        :param chat_id: Unique identifier for the target private chat
        :type chat_id: :obj:`int` or :obj:`str`

        :param title: Product name, 1-32 characters
        :type title: :obj:`str`

        :param description: Product description, 1-255 characters
        :type description: :obj:`str`

        :param invoice_payload: Bot-defined invoice payload, 1-128 bytes. This will not be displayed to the user,
            use for your internal processes.
        :type invoice_payload: :obj:`str`

        :param provider_token: Payments provider token, obtained via @Botfather; Pass None to omit the parameter
            to use "XTR" currency
        :type provider_token: :obj:`str`

        :param currency: Three-letter ISO 4217 currency code,
            see https://core.telegram.org/bots/payments#supported-currencies
        :type currency: :obj:`str`

        :param prices: Price breakdown, a list of components
            (e.g. product price, tax, discount, delivery cost, delivery tax, bonus, etc.)
        :type prices: List[:obj:`tgram.types.LabeledPrice`]

        :param start_parameter: Unique deep-linking parameter that can be used to generate this invoice
            when used as a start parameter
        :type start_parameter: :obj:`str`

        :param photo_url: URL of the product photo for the invoice. Can be a photo of the goods
            or a marketing image for a service. People like it better when they see what they are paying for.
        :type photo_url: :obj:`str`

        :param photo_size: Photo size in bytes
        :type photo_size: :obj:`int`

        :param photo_width: Photo width
        :type photo_width: :obj:`int`

        :param photo_height: Photo height
        :type photo_height: :obj:`int`

        :param need_name: Pass True, if you require the user's full name to complete the order
        :type need_name: :obj:`bool`

        :param need_phone_number: Pass True, if you require the user's phone number to complete the order
        :type need_phone_number: :obj:`bool`

        :param need_email: Pass True, if you require the user's email to complete the order
        :type need_email: :obj:`bool`

        :param need_shipping_address: Pass True, if you require the user's shipping address to complete the order
        :type need_shipping_address: :obj:`bool`

        :param is_flexible: Pass True, if the final price depends on the shipping method
        :type is_flexible: :obj:`bool`

        :param send_phone_number_to_provider: Pass True, if user's phone number should be sent to provider
        :type send_phone_number_to_provider: :obj:`bool`

        :param send_email_to_provider: Pass True, if user's email address should be sent to provider
        :type send_email_to_provider: :obj:`bool`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param reply_markup: A JSON-serialized object for an inline keyboard. If empty,
            one 'Pay total price' button will be shown. If not empty, the first button must be a Pay button
        :type reply_markup: :obj:`str`

        :param provider_data: A JSON-serialized data about the invoice, which will be shared with the payment provider.
            A detailed description of required fields should be provided by the payment provider.
        :type provider_data: :obj:`str`

        :param timeout: Timeout of a request, defaults to None
        :type timeout: :obj:`int`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the message should be sent even if the specified replied-to message is not found
        :type allow_sending_without_reply: :obj:`bool`

        :param max_tip_amount: The maximum accepted amount for tips in the smallest units of the currency
        :type max_tip_amount: :obj:`int`

        :param suggested_tip_amounts: A JSON-serialized array of suggested amounts of tips in the smallest
            units of the currency.  At most 4 suggested tip amounts can be specified. The suggested tip
            amounts must be positive, passed in a strictly increased order and must not exceed max_tip_amount.
        :type suggested_tip_amounts: :obj:`list` of :obj:`int`

        :param protect_content: Protects the contents of the sent message from forwarding and saving
        :type protect_content: :obj:`bool`

        :param message_thread_id: The identifier of a message thread, in which the invoice message will be sent
        :type message_thread_id: :obj:`int`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param message_effect_id: The identifier of a message effect to be applied to the message
        :type message_effect_id: :obj:`str`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :obj:`tgram.types.Message`
        """

        result = await self(
            "sendInvoice",
            chat_id=chat_id,
            title=title,
            description=description,
            payload=payload,
            currency=currency,
            prices=prices,
            message_thread_id=message_thread_id,
            provider_token=provider_token,
            max_tip_amount=max_tip_amount,
            suggested_tip_amounts=suggested_tip_amounts,
            start_parameter=start_parameter,
            provider_data=provider_data,
            photo_url=photo_url,
            photo_size=photo_size,
            photo_width=photo_width,
            photo_height=photo_height,
            need_name=need_name,
            need_phone_number=need_phone_number,
            need_email=need_email,
            need_shipping_address=need_shipping_address,
            send_phone_number_to_provider=send_phone_number_to_provider,
            send_email_to_provider=send_email_to_provider,
            is_flexible=is_flexible,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
from typing import Union
from tgram.types import ChatPermissions


class SetChatPermissions:
    async def set_chat_permissions(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        permissions: ChatPermissions,
        use_independent_chat_permissions: bool = None,
    ) -> bool:
        """
        Use this method to set default chat permissions for all members.
        The bot must be an administrator in the group or a supergroup for this to work
        and must have the can_restrict_members admin rights.

        Telegram documentation: https://core.telegram.org/bots/api#setchatpermissions

        :param chat_id: Unique identifier for the target chat or username of the target supergroup
            (in the format @supergroupusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param permissions: New default chat permissions
        :type permissions: :class:`tgram.types..ChatPermissions`

        :param use_independent_chat_permissions: Pass True if chat permissions are set independently. Otherwise,
            the can_send_other_messages and can_add_web_page_previews permissions will imply the can_send_messages,
            can_send_audios, can_send_documents, can_send_photos, can_send_videos, can_send_video_notes, and
            can_send_voice_notes permissions; the can_send_polls permission will imply the can_send_messages permission.
        :type use_independent_chat_permissions: :obj:`bool`

        :return: True on success
        :rtype: :obj:`bool`
        """

        result = await self(
            "setChatPermissions",
            chat_id=chat_id,
            permissions=permissions,
            use_independent_chat_permissions=use_independent_chat_permissions,
        )
        return result["result"]



import tgram


class RemoveUserVerification:
    async def remove_user_verification(
        self: "tgram.TgBot",
        user_id: int,
    ) -> bool:
        """
        Removes verification from a user who is currently verified on behalf of the organization represented by the bot. Returns True on success.
        Telegram documentation: https://core.telegram.org/bots/api#removeuserverification

        :param user_id: Unique identifier of the target user
        :type user_id: :obj:`int`

        :return: True on success
        :rtype: :obj:`bool`
        """

        result = await self(
            "removeUserVerification",
            user_id=user_id,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import Optional


class ProximityAlertTriggered(Type_):
    """
    This object represents the content of a service message, sent whenever a user in the chat triggers a proximity alert set by another user.

    Telegram Documentation: https://core.telegram.org/bots/api#proximityalerttriggered

    :param traveler: User that triggered the alert
    :type traveler: :class:`tgram.types.User`

    :param watcher: User that set the alert
    :type watcher: :class:`tgram.types.User`

    :param distance: The distance between the users
    :type distance: :obj:`int`

    :return: Instance of the class
    :rtype: :class:`tgram.types.ProximityAlertTriggered`
    """

    def __init__(
        self,
        traveler: "tgram.types.User" = None,
        watcher: "tgram.types.User" = None,
        distance: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.traveler = traveler
        self.watcher = watcher
        self.distance = distance

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.ProximityAlertTriggered"]:
        return (
            ProximityAlertTriggered(
                me=me,
                json=d,
                traveler=tgram.types.User._parse(me=me, d=d.get("traveler")),
                watcher=tgram.types.User._parse(me=me, d=d.get("watcher")),
                distance=d.get("distance"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



__all__ = [
    "types",
    "TgBot",
    "handlers",
    "filters",
    "compose",
    "StopPropagation",
    "ContinuePropagation",
    "utils",
    "storage",
]

__version__ = "2.0.0"

from . import (
    filters,
    utils,
    storage,
    handlers,
    types,
)

from .client import TgBot
from .sync import compose
from .errors import StopPropagation, ContinuePropagation



import tgram

from typing import Union


class RemoveChatVerification:
    async def remove_chat_verification(
        self: "tgram.TgBot",
        chat_id: Union[str, int],
    ) -> bool:
        """
        Removes verification from a chat that is currently verified on behalf of the organization represented by the bot. Returns True on success.
        Telegram documentation: https://core.telegram.org/bots/api#removechatverification

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :return: True on success
        :rtype: :obj:`bool`
        """

        result = await self(
            "removeChatVerification",
            chat_id=chat_id,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import Optional


class RevenueWithdrawalStateSucceeded(Type_):
    """
    The withdrawal succeeded.

    Telegram documentation: https://core.telegram.org/bots/api#revenuewithdrawalstatesucceeded

    :param type: Type of the state, always “succeeded”
    :type type: :obj:`str`

    :param date: Date the withdrawal was completed in Unix time
    :type date: :obj:`int`

    :param url: An HTTPS URL that can be used to see transaction details
    :type url: :obj:`str`

    :return: Instance of the class
    :rtype: :class:`RevenueWithdrawalStateSucceeded`
    """

    def __init__(
        self,
        date: "int" = None,
        url: "str" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.type = "succeeded"
        self.date = date
        self.url = url

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.RevenueWithdrawalStateSucceeded"]:
        return (
            RevenueWithdrawalStateSucceeded(
                me=me, json=d, type=d.get("type"), date=d.get("date"), url=d.get("url")
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import Union


class ReopenForumTopic:
    async def reopen_forum_topic(
        self: "tgram.TgBot", chat_id: Union[int, str], message_thread_id: int
    ) -> bool:
        """
        Use this method to reopen a closed topic in a forum supergroup chat. The bot must be an administrator in the chat
        for this to work and must have the can_manage_topics administrator rights, unless it is the creator of the topic.
        Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#reopenforumtopic

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param message_thread_id: Identifier of the topic to reopen
        :type message_thread_id: :obj:`int`

        :return: On success, True is returned.
        :rtype: :obj:`bool`
        """

        result = await self(
            "reopenForumTopic",
            chat_id=chat_id,
            message_thread_id=message_thread_id,
        )
        return result["result"]



from .async_property import AsyncProperty
from .compose import compose
from .file_id import decode_file_id, FILE_TYPES, SUPPORTED_FILE_TYPES_TO_SEND
from .files import get_file_name, get_file_path
from .json_ import Json
from .readable_time import convert_timestamp, ReadableTime
from .mention import Mention
from .parsers import markdown_unparse, html_unparse, String
from .parse_mode import get_parse_mode
from .types import (
    message_origin_parse,
    convert_input_media,
    reaction_type_parse,
    convert_to_inline_keyboard_markup,
)

from ..handlers import Handlers
from typing import List

API_URL = "https://api.telegram.org/"
ALL_UPDATES: List[str] = [
    getattr(Handlers, i)
    for i in filter(lambda x: not x.startswith("_"), Handlers.__dict__)
]

__all__ = [
    "AsyncProperty",
    "compose",
    "decode_file_id",
    "FILE_TYPES",
    "SUPPORTED_FILE_TYPES_TO_SEND",
    "get_file_name",
    "get_file_path",
    "Json",
    "Mention",
    "markdown_unparse",
    "html_unparse",
    "String",
    "message_origin_parse",
    "convert_input_media",
    "convert_timestamp",
    "ReadableTime",
    "get_parse_mode",
    "reaction_type_parse",
    "convert_to_inline_keyboard_markup",
]



import tgram
from typing import Union
from tgram.types import InlineKeyboardMarkup
from tgram.types import InputMedia
from tgram.types import Message

from tgram.utils import convert_input_media, convert_to_inline_keyboard_markup


class EditMessageMedia:
    async def edit_message_media(
        self: "tgram.TgBot",
        media: InputMedia,
        business_connection_id: str = None,
        chat_id: Union[int, str] = None,
        message_id: int = None,
        inline_message_id: str = None,
        reply_markup: InlineKeyboardMarkup = None,
    ) -> Union[Message, bool]:
        """
        Use this method to edit animation, audio, document, photo, or video messages, or to add media to text messages.
        If a message is part of a message album, then it can be edited only to an audio for audio albums, only to a document for document albums and to a photo or a video otherwise.
        When an inline message is edited, a new file can't be uploaded; use a previously uploaded file via its file_id or specify a URL.
        On success, if the edited message is not an inline message, the edited Message is returned, otherwise True is returned.

        Note that business messages that were not sent by the bot and do not contain an inline keyboard can only be edited within 48 hours from the time they were sent.
        """
        converted, file = convert_input_media([media])
        result = await self(
            "editMessageMedia",
            media=converted[0],
            business_connection_id=business_connection_id,
            chat_id=chat_id,
            message_id=message_id,
            inline_message_id=inline_message_id,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            **file,
        )
        return (
            Message._parse(me=self, d=result["result"])
            if isinstance(result["result"], dict)
            else result["result"]
        )



import tgram
from .type_ import Type_

from typing import Optional


class BackgroundFillGradient(Type_):
    """
    The background is a gradient fill.

    Telegram documentation: https://core.telegram.org/bots/api#backgroundfillgradient

    :param type: Type of the background fill, always “gradient”
    :type type: :obj:`str`

    :param top_color: Top color of the gradient in the RGB24 format
    :type top_color: :class:`int`

    :param bottom_color: Bottom color of the gradient in the RGB24 format
    :type bottom_color: :class:`int`

    :param rotation_angle: Clockwise rotation angle of the background fill in degrees; 0-359
    :type rotation_angle: :class:`int`

    :return: Instance of the class
    :rtype: :class:`BackgroundFillGradient`
    """

    def __init__(
        self,
        type: "str" = None,
        top_color: "int" = None,
        bottom_color: "int" = None,
        rotation_angle: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.type = type
        self.top_color = top_color
        self.bottom_color = bottom_color
        self.rotation_angle = rotation_angle

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.BackgroundFillGradient"]:
        return (
            BackgroundFillGradient(
                me=me,
                json=d,
                type=d.get("type"),
                top_color=d.get("top_color"),
                bottom_color=d.get("bottom_color"),
                rotation_angle=d.get("rotation_angle"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram


class TransferGift:
    async def transfer_gift(
        self: "tgram.TgBot",
        business_connection_id: str,
        owned_gift_id: str,
        new_owner_chat_id: int,
        star_count: int = None,
    ) -> bool:
        """
        Transfers an owned unique gift to another user. Requires the can_transfer_and_upgrade_gifts business bot right.
        Requires can_transfer_stars business bot right if the transfer is paid. Returns True on success.

        Parameters:
            business_connection_id (str): Unique identifier of the business connection.
            owned_gift_id (str): Unique identifier of the regular gift that should be transferred.
            new_owner_chat_id (int): Unique identifier of the chat which will own the gift. The chat must be active in the last 24 hours.
            star_count (int, optional): The amount of Telegram Stars that will be paid for the transfer from the business account balance. If positive, then the can_transfer_stars business bot right is required.
        """
        result = await self(
            "transferGift",
            business_connection_id=business_connection_id,
            owned_gift_id=owned_gift_id,
            new_owner_chat_id=new_owner_chat_id,
            star_count=star_count,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import Union, Optional


class ChatMember(Type_):
    """
    This object contains information about one member of a chat.
    Currently, the following 6 types of chat members are supported:

    * :class:`tgram.types.ChatMemberOwner`
    * :class:`tgram.types.ChatMemberAdministrator`
    * :class:`tgram.types.ChatMemberMember`
    * :class:`tgram.types.ChatMemberRestricted`
    * :class:`tgram.types.ChatMemberLeft`
    * :class:`tgram.types.ChatMemberBanned`

    Telegram Documentation: https://core.telegram.org/bots/api#chatmember
    """

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional[
        Union[
            "tgram.types.ChatMemberOwner",
            "tgram.types.ChatMemberAdministrator",
            "tgram.types.ChatMemberMember",
            "tgram.types.ChatMemberRestricted",
            "tgram.types.ChatMemberBanned",
            "tgram.types.ChatMemberLeft",
        ]
    ]:
        return (
            (
                tgram.types.ChatMemberOwner._parse(me=me, d=d)
                if d.get("status") == "creator"
                else tgram.types.ChatMemberAdministrator._parse(me=me, d=d)
                if d.get("status") == "administrator"
                else tgram.types.ChatMemberMember._parse(me=me, d=d)
                if d.get("status") == "member"
                else tgram.types.ChatMemberRestricted._parse(me=me, d=d)
                if d.get("status") == "restricted"
                else tgram.types.ChatMemberLeft._parse(me=me, d=d)
                if d.get("status") == "left"
                else tgram.types.ChatMemberBanned._parse(me=me, d=d)
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from tgram.types import User


class GetMe:
    async def get_me(self: "tgram.TgBot") -> User:
        """
        Returns basic information about the bot in form of a User object.

        Telegram documentation: https://core.telegram.org/bots/api#getme
        """

        result = await self(
            "getMe",
        )
        return User._parse(me=self, d=result["result"])



import tgram

from .type_ import Type_
from typing import List, Optional


class GiftInfo(Type_):
    """
    Describes a service message about a regular gift that was sent or received.

    :param gift: Information about the gift
    :type gift: :class:`tgram.types.Gift`
    :param owned_gift_id: Optional. Unique identifier of the received gift for the bot; only present for gifts received on behalf of business accounts
    :type owned_gift_id: :obj:`str`
    :param convert_star_count: Optional. Number of Telegram Stars that can be claimed by the receiver by converting the gift; omitted if conversion to Telegram Stars is impossible
    :type convert_star_count: :obj:`int`
    :param prepaid_upgrade_star_count: Optional. Number of Telegram Stars that were prepaid by the sender for the ability to upgrade the gift
    :type prepaid_upgrade_star_count: :obj:`int`
    :param can_be_upgraded: Optional. True, if the gift can be upgraded to a unique gift
    :type can_be_upgraded: :obj:`bool`
    :param text: Optional. Text of the message that was added to the gift
    :type text: :obj:`str`
    :param entities: Optional. Special entities that appear in the text
    :type entities: List[:class:`tgram.types.MessageEntity`]
    :param is_private: Optional. True, if the sender and gift text are shown only to the gift receiver; otherwise, everyone will be able to see them
    :type is_private: :obj:`bool`
    """

    def __init__(
        self,
        gift: "tgram.types.Gift" = None,
        owned_gift_id: "str" = None,
        convert_star_count: "int" = None,
        prepaid_upgrade_star_count: "int" = None,
        can_be_upgraded: "bool" = None,
        text: "str" = None,
        entities: List["tgram.types.MessageEntity"] = None,
        is_private: "bool" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.gift = gift
        self.owned_gift_id = owned_gift_id
        self.convert_star_count = convert_star_count
        self.prepaid_upgrade_star_count = prepaid_upgrade_star_count
        self.can_be_upgraded = can_be_upgraded
        self.text = text
        self.entities = entities
        self.is_private = is_private

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["GiftInfo"]:
        return (
            GiftInfo(
                gift=tgram.types.Gift._parse(me, d.get("gift")),
                owned_gift_id=d.get("owned_gift_id"),
                convert_star_count=d.get("convert_star_count"),
                prepaid_upgrade_star_count=d.get("prepaid_upgrade_star_count"),
                can_be_upgraded=d.get("can_be_upgraded"),
                text=d.get("text"),
                entities=[
                    tgram.types.MessageEntity._parse(me, ent)
                    for ent in d.get("entities", [])
                ]
                if d.get("entities")
                else None,
                is_private=d.get("is_private"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import ReplyParameters
from tgram.types import InputChecklist

from tgram.utils import convert_to_inline_keyboard_markup


class SendChecklist:
    async def send_checklist(
        self: "tgram.TgBot",
        business_connection_id: str,
        chat_id: int,
        checklist: InputChecklist,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: InlineKeyboardMarkup = None,
    ) -> Message:
        """
        Use this method to send a checklist on behalf of a connected business account.
        On success, the sent Message is returned.

        Telegram documentation: https://core.telegram.org/bots/api#sendchecklist

        :param business_connection_id: Unique identifier of the business connection on behalf of which the message will be sent
        :type business_connection_id: :obj:`str`

        :param chat_id: Unique identifier for the target chat
        :type chat_id: :obj:`int`

        :param checklist: A JSON-serialized object for the checklist to send
        :type checklist: :class:`tgram.types.InputChecklist`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound
        :type disable_notification: :obj:`bool`

        :param protect_content: Protects the contents of the sent message from forwarding and saving
        :type protect_content: :obj:`bool`

        :param message_effect_id: Unique identifier of the message effect to be added to the message
        :type message_effect_id: :obj:`str`

        :param reply_parameters: A JSON-serialized object for description of the message to reply to
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param reply_markup: A JSON-serialized object for an inline keyboard
        :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup`

        :return: On success, the sent Message is returned.
        :rtype: :class:`tgram.types.Message`
        """
        result = await self(
            "sendChecklist",
            business_connection_id=business_connection_id,
            chat_id=chat_id,
            checklist=checklist,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
from .type_ import Type_

from typing import Optional


class BotCommandScopeDefault(Type_):
    """
    Represents the default scope of bot commands. Default commands are used if no commands with a narrower scope are specified for the user.

    Telegram Documentation: https://core.telegram.org/bots/api#botcommandscopedefault

    :param type: Scope type, must be default
    :type type: :obj:`str`

    :return: Instance of the class
    :rtype: :class:`tgram.types.BotCommandScopeDefault`
    """

    def __init__(
        self, type: "str" = None, me: "tgram.TgBot" = None, json: "dict" = None
    ):
        super().__init__(me=me, json=json)
        self.type = type

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.BotCommandScopeDefault"]:
        return (
            BotCommandScopeDefault(me=me, json=d, type=d.get("type"))
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from .type_ import Type_

from typing import Optional


class KeyboardButton(Type_):
    """
    This object represents one button of the reply keyboard. For simple text buttons String can be used instead of this object to specify text of the button. Optional fields web_app, request_contact, request_location, and request_poll are mutually exclusive.

    Telegram Documentation: https://core.telegram.org/bots/api#keyboardbutton

    :param text: Text of the button. If none of the optional fields are used, it will be sent as a message when the button is
        pressed
    :type text: :obj:`str`

    :param request_contact: Optional. If True, the user's phone number will be sent as a contact when the button is
        pressed. Available in private chats only.
    :type request_contact: :obj:`bool`

    :param request_location: Optional. If True, the user's current location will be sent when the button is pressed.
        Available in private chats only.
    :type request_location: :obj:`bool`

    :param request_poll: Optional. If specified, the user will be asked to create a poll and send it to the bot when the
        button is pressed. Available in private chats only.
    :type request_poll: :class:`tgram.types.KeyboardButtonPollType`

    :param web_app: Optional. If specified, the described Web App will be launched when the button is pressed. The Web App
        will be able to send a “web_app_data” service message. Available in private chats only.
    :type web_app: :class:`tgram.types.WebAppInfo`

    :param request_user: deprecated
    :type request_user: :class:`tgram.types.KeyboardButtonRequestUser`

    :param request_users: Optional. If specified, pressing the button will open a list of suitable users.
        Identifiers of selected users will be sent to the bot in a “users_shared” service message. Available in private chats only.
    :type request_users: :class:`tgram.types.KeyboardButtonRequestUsers`

    :param request_chat: Optional. If specified, pressing the button will open a list of suitable chats. Tapping on a chat will
        send its identifier to the bot in a “chat_shared” service message. Available in private chats only.
    :type request_chat: :class:`tgram.types.KeyboardButtonRequestChat`

    :return: Instance of the class
    :rtype: :class:`tgram.types.KeyboardButton`
    """

    def __init__(
        self,
        text: "str" = None,
        request_users: "tgram.types.KeyboardButtonRequestUsers" = None,
        request_chat: "tgram.types.KeyboardButtonRequestChat" = None,
        request_contact: "bool" = None,
        request_location: "bool" = None,
        request_poll: "tgram.types.KeyboardButtonPollType" = None,
        web_app: "tgram.types.WebAppInfo" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.text = text
        self.request_users = request_users
        self.request_chat = request_chat
        self.request_contact = request_contact
        self.request_location = request_location
        self.request_poll = request_poll
        self.web_app = web_app

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.KeyboardButton"]:
        return (
            KeyboardButton(
                me=me,
                json=d,
                text=d.get("text"),
                request_users=tgram.types.KeyboardButtonRequestUsers._parse(
                    me=me, d=d.get("request_users")
                ),
                request_chat=tgram.types.KeyboardButtonRequestChat._parse(
                    me=me, d=d.get("request_chat")
                ),
                request_contact=d.get("request_contact"),
                request_location=d.get("request_location"),
                request_poll=tgram.types.KeyboardButtonPollType._parse(
                    me=me, d=d.get("request_poll")
                ),
                web_app=tgram.types.WebAppInfo._parse(me=me, d=d.get("web_app")),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from tgram.types import StarAmount


class GetBusinessAccountStarBalance:
    async def get_business_account_star_balance(
        self: "tgram.TgBot", business_connection_id: str
    ) -> StarAmount:
        """
        Returns the amount of Telegram Stars owned by a managed business account.

        Telegram documentation: https://core.telegram.org/bots/api#getbusinessaccountstarbalance

        :param business_connection_id: Unique identifier of the business connection
        :return: StarAmount object
        """
        result = await self(
            "getBusinessAccountStarBalance",
            business_connection_id=business_connection_id,
        )
        return StarAmount._parse(me=self, d=result["result"])



import tgram
from .type_ import Type_

from typing import List, Optional
from tgram.utils import message_origin_parse


class ExternalReplyInfo(Type_):
    """
    This object contains information about a message that is being replied to,
    which may come from another chat or forum topic.

    Telegram documentation: https://core.telegram.org/bots/api#externalreplyinfo

    :param origin: Origin of the message replied to by the given message
    :type origin: :class:`MessageOrigin`

    :param chat: Optional. Chat the original message belongs to. Available only if the chat is a supergroup or a channel.
    :type chat: :class:`Chat`

    :param message_id: Optional. Unique message identifier inside the original chat. Available only if the original chat is a supergroup or a channel.
    :type message_id: :obj:`int`

    :param link_preview_options: Optional. Options used for link preview generation for the original message, if it is a text message
    :type link_preview_options: :class:`LinkPreviewOptions`

    :param animation: Optional. Message is an animation, information about the animation
    :type animation: :class:`Animation`

    :param audio: Optional. Message is an audio file, information about the file
    :type audio: :class:`Audio`

    :param document: Optional. Message is a general file, information about the file
    :type document: :class:`Document`

    :param photo: Optional. Message is a photo, available sizes of the photo
    :type photo: :obj:`list` of :class:`PhotoSize`

    :param sticker: Optional. Message is a sticker, information about the sticker
    :type sticker: :class:`Sticker`

    :param story: Optional. Message is a forwarded story
    :type story: :class:`Story`

    :param video: Optional. Message is a video, information about the video
    :type video: :class:`Video`

    :param video_note: Optional. Message is a video note, information about the video message
    :type video_note: :class:`VideoNote`

    :param voice: Optional. Message is a voice message, information about the file
    :type voice: :class:`Voice`

    :param has_media_spoiler: Optional. True, if the message media is covered by a spoiler animation
    :type has_media_spoiler: :obj:`bool`

    :param checklist: Optional. Message is a checklist
    :type checklist: :class:`tgram.types.Checklist

    :param contact: Optional. Message is a shared contact, information about the contact
    :type contact: :class:`Contact`

    :param dice: Optional. Message is a dice with random value
    :type dice: :class:`Dice`

    :param game: Optional. Message is a game, information about the game. More about games »
    :type game: :class:`Game`

    :param giveaway: Optional. Message is a scheduled giveaway, information about the giveaway
    :type giveaway: :class:`Giveaway`

    :param giveaway_winners: Optional. A giveaway with public winners was completed
    :type giveaway_winners: :class:`GiveawayWinners`

    :param invoice: Optional. Message is an invoice for a payment, information about the invoice. More about payments »
    :type invoice: :class:`Invoice`

    :param location: Optional. Message is a shared location, information about the location
    :type location: :class:`Location`

    :param poll: Optional. Message is a native poll, information about the poll
    :type poll: :class:`Poll`

    :param venue: Optional. Message is a venue, information about the venue
    :type venue: :class:`Venue`

    :return: Instance of the class
    :rtype: :class:`ExternalReplyInfo`
    """

    def __init__(
        self,
        origin: "tgram.types.MessageOrigin" = None,
        chat: "tgram.types.Chat" = None,
        message_id: "int" = None,
        link_preview_options: "tgram.types.LinkPreviewOptions" = None,
        animation: "tgram.types.Animation" = None,
        audio: "tgram.types.Audio" = None,
        document: "tgram.types.Document" = None,
        paid_media: "tgram.types.PaidMediaInfo" = None,
        photo: List["tgram.types.PhotoSize"] = None,
        sticker: "tgram.types.Sticker" = None,
        story: "tgram.types.Story" = None,
        video: "tgram.types.Video" = None,
        video_note: "tgram.types.VideoNote" = None,
        voice: "tgram.types.Voice" = None,
        has_media_spoiler: "bool" = None,
        checklist: "tgram.types.Checklist" = None,
        contact: "tgram.types.Contact" = None,
        dice: "tgram.types.Dice" = None,
        game: "tgram.types.Game" = None,
        giveaway: "tgram.types.Giveaway" = None,
        giveaway_winners: "tgram.types.GiveawayWinners" = None,
        invoice: "tgram.types.Invoice" = None,
        location: "tgram.types.Location" = None,
        poll: "tgram.types.Poll" = None,
        venue: "tgram.types.Venue" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.origin = origin
        self.chat = chat
        self.message_id = message_id
        self.link_preview_options = link_preview_options
        self.animation = animation
        self.audio = audio
        self.document = document
        self.paid_media = paid_media
        self.photo = photo
        self.sticker = sticker
        self.story = story
        self.video = video
        self.video_note = video_note
        self.voice = voice
        self.has_media_spoiler = has_media_spoiler
        self.checklist = checklist
        self.contact = contact
        self.dice = dice
        self.game = game
        self.giveaway = giveaway
        self.giveaway_winners = giveaway_winners
        self.invoice = invoice
        self.location = location
        self.poll = poll
        self.venue = venue

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.ExternalReplyInfo"]:
        return (
            ExternalReplyInfo(
                me=me,
                json=d,
                origin=message_origin_parse(d.get("origin"), me),
                chat=tgram.types.Chat._parse(me=me, d=d.get("chat")),
                message_id=d.get("message_id"),
                link_preview_options=tgram.types.LinkPreviewOptions._parse(
                    me=me, d=d.get("link_preview_options")
                ),
                animation=tgram.types.Animation._parse(me=me, d=d.get("animation")),
                audio=tgram.types.Audio._parse(me=me, d=d.get("audio")),
                document=tgram.types.Document._parse(me=me, d=d.get("document")),
                paid_media=tgram.types.PaidMediaInfo._parse(
                    me=me, d=d.get("paid_media")
                ),
                photo=[tgram.types.PhotoSize._parse(me=me, d=i) for i in d.get("photo")]
                if d.get("photo")
                else None,
                sticker=tgram.types.Sticker._parse(me=me, d=d.get("sticker")),
                story=tgram.types.Story._parse(me=me, d=d.get("story")),
                video=tgram.types.Video._parse(me=me, d=d.get("video")),
                video_note=tgram.types.VideoNote._parse(me=me, d=d.get("video_note")),
                voice=tgram.types.Voice._parse(me=me, d=d.get("voice")),
                has_media_spoiler=d.get("has_media_spoiler"),
                checklist=tgram.types.Checklist._parse(me=me, d=d.get("checklist")),
                contact=tgram.types.Contact._parse(me=me, d=d.get("contact")),
                dice=tgram.types.Dice._parse(me=me, d=d.get("dice")),
                game=tgram.types.Game._parse(me=me, d=d.get("game")),
                giveaway=tgram.types.Giveaway._parse(me=me, d=d.get("giveaway")),
                giveaway_winners=tgram.types.GiveawayWinners._parse(
                    me=me, d=d.get("giveaway_winners")
                ),
                invoice=tgram.types.Invoice._parse(me=me, d=d.get("invoice")),
                location=tgram.types.Location._parse(me=me, d=d.get("location")),
                poll=tgram.types.Poll._parse(me=me, d=d.get("poll")),
                venue=tgram.types.Venue._parse(me=me, d=d.get("venue")),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



from .callback_query import CallbackB
from .chat_join_request import ChatJoinRequestB
from .chat_member_updated import ChatMemberUpdatedB
from .chat import ChatB
from .chosen_inline_result import ChosenInlineResultB
from .inline_query import InlineQueryB
from .message import MessageB
from .paid_media_purchased import PaidMediaPurchasedB
from .pre_checkout_query import PreCheckoutQueryB
from .shipping_query import ShippingQueryB
from .user import UserB

from tgram import sync

import inspect
import sys

for name, obj in inspect.getmembers(sys.modules[__name__]):
    if inspect.isclass(obj):
        sync.wrap(obj)

__all__ = [
    "CallbackB",
    "ChatJoinRequestB",
    "ChatMemberUpdatedB",
    "ChatB",
    "ChosenInlineResultB",
    "InlineQueryB",
    "MessageB",
    "PaidMediaPurchasedB",
    "PreCheckoutQueryB",
    "ShippingQueryB",
    "UserB",
]



import tgram
from tgram.types import MenuButton


class SetChatMenuButton:
    async def set_chat_menu_button(
        self: "tgram.TgBot", chat_id: int = None, menu_button: MenuButton = None
    ) -> bool:
        """
        Use this method to change the bot's menu button in a private chat,
        or the default menu button.
        Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#setchatmenubutton

        :param chat_id: Unique identifier for the target private chat.
            If not specified, default bot's menu button will be changed.
        :type chat_id: :obj:`int` or :obj:`str`

        :param menu_button: A JSON-serialized object for the new bot's menu button. Defaults to MenuButtonDefault
        :type menu_button: :class:`tgram.types.MenuButton`

        :return: True on success.
        :rtype: :obj:`bool`
        """

        result = await self(
            "setChatMenuButton",
            chat_id=chat_id,
            menu_button=menu_button,
        )
        return result["result"]



import tgram
from typing import Union


class BanChatSenderChat:
    async def ban_chat_sender_chat(
        self: "tgram.TgBot", chat_id: Union[int, str], sender_chat_id: int
    ) -> bool:
        """
        Use this method to ban a channel chat in a supergroup or a channel.
        The owner of the chat will not be able to send messages and join live
        streams on behalf of the chat, unless it is unbanned first.
        The bot must be an administrator in the supergroup or channel
        for this to work and must have the appropriate administrator rights.
        Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#banchatsenderchat

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param sender_chat_id: Unique identifier of the target sender chat
        :type sender_chat_id: :obj:`int` or :obj:`str`

        :return: True on success.
        :rtype: :obj:`bool`
        """

        result = await self(
            "banChatSenderChat", chat_id=chat_id, sender_chat_id=sender_chat_id
        )
        return result["result"]



import tgram

from typing import List


class GiftPremiumSubscription:
    async def gift_premium_subscription(
        self: "tgram.TgBot",
        user_id: int,
        month_count: int,
        star_count: int,
        text: str = None,
        text_parse_mode: str = None,
        text_entities: List["tgram.types.MessageEntity"] = None,
    ) -> bool:
        """
        Gifts a Telegram Premium subscription to the given user.
        Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#giftpremiumsubscription

        :param user_id: Unique identifier of the target user who will receive a Telegram Premium subscription
        :type user_id: :obj:`int`

        :param month_count: Number of months the Telegram Premium subscription will be active for the user; must be one of 3, 6, or 12
        :type month_count: :obj:`int`

        :param star_count: Number of Telegram Stars to pay for the Telegram Premium subscription; must be 1000 for 3 months, 1500 for 6 months, and 2500 for 12 months
        :type star_count: :obj:`int`

        :param text: Text that will be shown along with the service message about the subscription; 0-128 characters
        :type text: :obj:`str`, optional

        :param text_parse_mode: Mode for parsing entities in the text
        :type text_parse_mode: :obj:`str`, optional

        :param text_entities: A JSON-serialized list of special entities that appear in the gift text
        :type text_entities: :obj:`list`, optional

        :return: True on success.
        :rtype: :obj:`bool`
        """
        result = await self(
            "giftPremiumSubscription",
            user_id=user_id,
            month_count=month_count,
            star_count=star_count,
            text=text,
            text_parse_mode=text_parse_mode,
            text_entities=text_entities,
        )
        return result["result"]



import tgram

from typing import Optional


class ChatMemberUpdatedB:
    @property
    def user(self: "tgram.types.ChatMemberUpdated") -> Optional["tgram.types.User"]:
        """
        Get the user who sent the callback query.

        Returns:
            Optional[tgram.types.User]: The user who sent the callback query.
        """
        return self.from_user

    sender_user = user



import base64
import struct

from typing import List, TypedDict
from io import BytesIO

from tgram import utils


def b64_decode(s: str) -> bytes:
    return base64.urlsafe_b64decode(s + "=" * (-len(s) % 4))


def rle_decode(s: bytes) -> bytes:
    r: List[int] = []
    z: bool = False

    for b in s:
        if not b:
            z = True
            continue

        if z:
            r.extend((0,) * b)
            z = False
        else:
            r.append(b)

    return bytes(r)


FILE_TYPES = {
    0: "Thumbnail",
    1: "ChatPhoto",
    2: "Photo",
    3: "Voice",
    4: "Video",
    5: "Document",
    8: "Sticker",
    9: "Audio",
    10: "Animation",
    13: "VideoNote",
    17: "Document",
}

SUPPORTED_FILE_TYPES_TO_SEND = (2, 3, 4, 5, 8, 9, 10, 13, 17)

WEB_LOCATION_FLAG = 1 << 24
FILE_REFERENCE_FLAG = 1 << 25


class DecodedFileId(TypedDict):
    file_type_int: int
    file_type: str
    dc_id: int
    file_id: str


def decode_file_id(file_id: str) -> DecodedFileId:
    decoded = rle_decode(b64_decode(file_id))
    major = decoded[-1]

    if major < 4:
        buffer = BytesIO(decoded[:-1])
    else:
        buffer = BytesIO(decoded[:-2])

    file_type_int, dc_id = struct.unpack("<ii", buffer.read(8))

    file_type_int &= ~WEB_LOCATION_FLAG
    file_type_int &= ~FILE_REFERENCE_FLAG

    try:
        file_type = FILE_TYPES[file_type_int]
    except ValueError:
        raise ValueError(f"Unknown file_type {file_type} of file_id {file_id}")

    return utils.Json(
        {
            "file_type_int": file_type_int,
            "file_type": file_type,
            "dc_id": dc_id,
            "file_id": file_id,
        }
    )



import tgram
from .type_ import Type_

from typing import Optional


class AffiliateInfo(Type_):
    """
    This object represents an animation file (GIF or H.264/MPEG-4 AVC video without sound).

    Telegram Documentation: https://core.telegram.org/bots/api#affiliateinfo

    :param affiliate_user: Optional. The bot or the user that received an affiliate commission if it was received by a bot or a user
    :type affiliate_user: :class:`User`

    :param affiliate_chat: Optional. The chat that received an affiliate commission if it was received by a chat
    :type affiliate_chat: :class:`Chat`

    :param commission_per_mille: The number of Telegram Stars received by the affiliate for each 1000 Telegram Stars received by the bot from referred users
    :type commission_per_mille: :obj:`int`

    :param amount: Integer amount of Telegram Stars received by the affiliate from the transaction, rounded to 0; can be negative for refunds
    :type amount: :obj:`int`

    :param nanostar_amount: Optional. The number of 1/1000000000 shares of Telegram Stars received by the affiliate; from -999999999 to 999999999; can be negative for refunds
    :type nanostar_amount: :obj:`int`

    :return: Instance of the class
    :rtype: :class:`tgram.types.AffiliateInfo`
    """

    def __init__(
        self,
        affiliate_user: "tgram.types.User" = None,
        affiliate_chat: "tgram.types.Chat" = None,
        commission_per_mille: int = None,
        amount: int = None,
        nanostar_amount: int = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.affiliate_user = affiliate_user
        self.affiliate_chat = affiliate_chat
        self.commission_per_mille = commission_per_mille
        self.amount = amount
        self.nanostar_amount = nanostar_amount

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.AffiliateInfo"]:
        return (
            AffiliateInfo(
                me=me,
                json=d,
                affiliate_user=tgram.types.User._parse(
                    me=me, d=d.get("affiliate_user")
                ),
                affiliate_chat=tgram.types.Chat._parse(
                    me=me, d=d.get("affiliate_chat")
                ),
                commission_per_mille=d.get("commission_per_mille"),
                amount=d.get("amount"),
                nanostar_amount=d.get("nanostar_amount"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram

from typing import Optional


def get_parse_mode(
    bot: Optional["tgram.TgBot"] = None, parse_mode: Optional[str] = None
) -> Optional[str]:
    parse_mode = parse_mode or (bot.parse_mode if bot else None)

    if parse_mode and parse_mode.lower() in {"d", "disable", "disabled"}:
        return None
    return parse_mode



import tgram
from tgram.types import BotShortDescription


class GetMyShortDescription:
    async def get_my_short_description(
        self: "tgram.TgBot", language_code: str = None
    ) -> BotShortDescription:
        """
        Use this method to get the current bot short description for the given user language.
        Returns BotShortDescription on success.

        :param language_code: A two-letter ISO 639-1 language code or an empty string
        :type language_code: :obj:`str`

        :return: :class:`tgram.types.BotShortDescription`
        """

        result = await self(
            "getMyShortDescription",
            language_code=language_code,
        )
        return BotShortDescription._parse(me=self, d=result["result"])



from json import dumps


class Json(dict):
    def __str__(self) -> str:
        return dumps(
            self,
            ensure_ascii=False,
            indent=2,
            default=lambda obj: repr(obj) if not isinstance(obj, dict) else obj,
        )



import tgram
from .type_ import Type_

from typing import Optional
from tgram import bound


class InlineQuery(Type_, bound.InlineQueryB):
    """
    This object represents an incoming inline query. When the user sends an empty query, your bot could return some default or trending results.

    Telegram Documentation: https://core.telegram.org/bots/api#inlinequery

    :param id: Unique identifier for this query
    :type id: :obj:`str`

    :param from_user: Sender
    :type from_user: :class:`tgram.types.User`

    :param query: Text of the query (up to 256 characters)
    :type query: :obj:`str`

    :param offset: Offset of the results to be returned, can be controlled by the bot
    :type offset: :obj:`str`

    :param chat_type: Optional. Type of the chat from which the inline query was sent. Can be either “sender” for a private
        chat with the inline query sender, “private”, “group”, “supergroup”, or “channel”. The chat type should be always
        known for requests sent from official clients and most third-party clients, unless the request was sent from a secret
        chat
    :type chat_type: :obj:`str`

    :param location: Optional. Sender location, only for bots that request user location
    :type location: :class:`tgram.types.Location`

    :return: Instance of the class
    :rtype: :class:`tgram.types.InlineQuery`
    """

    def __init__(
        self,
        id: "str" = None,
        from_user: "tgram.types.User" = None,
        query: "str" = None,
        offset: "str" = None,
        chat_type: "tgram.types.ChatType" = None,
        location: "tgram.types.Location" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.id = id
        self.from_user = from_user
        self.query = query
        self.offset = offset
        self.chat_type = chat_type
        self.location = location

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.InlineQuery"]:
        return (
            InlineQuery(
                me=me,
                json=d,
                id=d.get("id"),
                from_user=tgram.types.User._parse(me=me, d=d.get("from")),
                query=d.get("query"),
                offset=d.get("offset"),
                chat_type=d.get("chat_type"),
                location=tgram.types.Location._parse(me=me, d=d.get("location")),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
import random
from .type_ import Type_

from typing import Optional


class InlineQueryResultContact(Type_):
    """
    Represents a contact with a phone number. By default, this contact will be sent by the user. Alternatively, you can use input_message_content to send a message with the specified content instead of the contact.

    Telegram Documentation: https://core.telegram.org/bots/api#inlinequeryresultcontact

    :param type: Type of the result, must be contact
    :type type: :obj:`str`

    :param id: Unique identifier for this result, 1-64 Bytes
    :type id: :obj:`str`

    :param phone_number: Contact's phone number
    :type phone_number: :obj:`str`

    :param first_name: Contact's first name
    :type first_name: :obj:`str`

    :param last_name: Optional. Contact's last name
    :type last_name: :obj:`str`

    :param vcard: Optional. Additional data about the contact in the form of a vCard, 0-2048 bytes
    :type vcard: :obj:`str`

    :param reply_markup: Optional. Inline keyboard attached to the message
    :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup`

    :param input_message_content: Optional. Content of the message to be sent instead of the contact
    :type input_message_content: :class:`tgram.types.InputMessageContent`

    :param thumbnail_url: Optional. Url of the thumbnail for the result
    :type thumbnail_url: :obj:`str`

    :param thumbnail_width: Optional. Thumbnail width
    :type thumbnail_width: :obj:`int`

    :param thumbnail_height: Optional. Thumbnail height
    :type thumbnail_height: :obj:`int`

    :return: Instance of the class
    :rtype: :class:`tgram.types.InlineQueryResultContact`
    """

    def __init__(
        self,
        phone_number: "str" = None,
        first_name: "str" = None,
        last_name: "str" = None,
        vcard: "str" = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
        input_message_content: "tgram.types.InputMessageContent" = None,
        thumbnail_url: "str" = None,
        thumbnail_width: "int" = None,
        thumbnail_height: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.type = "contact"
        self.id = random.randint(10000, 99999)
        self.phone_number = phone_number
        self.first_name = first_name
        self.last_name = last_name
        self.vcard = vcard
        self.reply_markup = reply_markup
        self.input_message_content = input_message_content
        self.thumbnail_url = thumbnail_url
        self.thumbnail_width = thumbnail_width
        self.thumbnail_height = thumbnail_height

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.InlineQueryResultContact"]:
        return (
            InlineQueryResultContact(
                me=me,
                json=d,
                phone_number=d.get("phone_number"),
                first_name=d.get("first_name"),
                last_name=d.get("last_name"),
                vcard=d.get("vcard"),
                reply_markup=tgram.types.InlineKeyboardMarkup._parse(
                    me=me, d=d.get("reply_markup")
                ),
                input_message_content=tgram.types.InputMessageContent._parse(
                    me=me, d=d.get("input_message_content")
                ),
                thumbnail_url=d.get("thumbnail_url"),
                thumbnail_width=d.get("thumbnail_width"),
                thumbnail_height=d.get("thumbnail_height"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram


class TransferBusinessAccountStars:
    async def transfer_business_account_stars(
        self: "tgram.TgBot",
        business_connection_id: str,
        star_count: int,
    ) -> bool:
        """
        Transfers Telegram Stars from the business account balance to the bot's balance.
        Requires the can_transfer_stars business bot right.

        Telegram documentation: https://core.telegram.org/bots/api#transferbusinessaccountstars

        :param business_connection_id: Unique identifier of the business connection
        :type business_connection_id: :obj:`str`

        :param star_count: Number of Telegram Stars to transfer; 1-10000
        :type star_count: :obj:`int`

        :return: On success, returns True.
        :rtype: :obj:`bool`

        :raises ValueError: If star_count is not in the range 1-10000.
        """
        if not (1 <= star_count <= 10000):
            raise ValueError("star_count must be between 1 and 10000")

        result = await self(
            "transferBusinessAccountStars",
            business_connection_id=business_connection_id,
            star_count=star_count,
        )
        return result["result"]



import tgram
from typing import Union


class EditForumTopic:
    async def edit_forum_topic(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        message_thread_id: int,
        name: str = None,
        icon_custom_emoji_id: str = None,
    ) -> bool:
        """
        Use this method to edit name and icon of a topic in a forum supergroup chat. The bot must be an
        administrator in the chat for this to work and must have can_manage_topics administrator rights,
        unless it is the creator of the topic. Returns True on success.

        Telegram Documentation: https://core.telegram.org/bots/api#editforumtopic

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param message_thread_id: Identifier of the topic to edit
        :type message_thread_id: :obj:`int`

        :param name: Optional, New name of the topic, 1-128 characters. If not specififed or empty,
            the current name of the topic will be kept
        :type name: :obj:`str`

        :param icon_custom_emoji_id: Optional, New unique identifier of the custom emoji shown as the topic icon.
            Use getForumTopicIconStickers to get all allowed custom emoji identifiers. Pass an empty string to remove the
            icon. If not specified, the current icon will be kept
        :type icon_custom_emoji_id: :obj:`str`

        :return: On success, True is returned.
        :rtype: :obj:`bool`
        """

        result = await self(
            "editForumTopic",
            chat_id=chat_id,
            message_thread_id=message_thread_id,
            name=name,
            icon_custom_emoji_id=icon_custom_emoji_id,
        )
        return result["result"]



import tgram
from .type_ import Type_
from typing import Optional


class AcceptedGiftTypes(Type_):
    """
    This object describes the types of gifts that can be gifted to a user or a chat.

    Telegram Documentation: https://core.telegram.org/bots/api#acceptedgifttypes

    :param unlimited_gifts: True, if unlimited regular gifts are accepted
    :type unlimited_gifts: :obj:`bool`

    :param limited_gifts: True, if limited regular gifts are accepted
    :type limited_gifts: :obj:`bool`

    :param unique_gifts: True, if unique gifts or gifts that can be upgraded to unique for free are accepted
    :type unique_gifts: :obj:`bool`

    :param premium_subscription: True, if a Telegram Premium subscription is accepted
    :type premium_subscription: :obj:`bool`

    :return: Instance of the class
    :rtype: :class:`tgram.types.AcceptedGiftTypes`
    """

    def __init__(
        self,
        unlimited_gifts: bool = None,
        limited_gifts: bool = None,
        unique_gifts: bool = None,
        premium_subscription: bool = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.unlimited_gifts = unlimited_gifts
        self.limited_gifts = limited_gifts
        self.unique_gifts = unique_gifts
        self.premium_subscription = premium_subscription

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.AcceptedGiftTypes"]:
        return (
            AcceptedGiftTypes(
                me=me,
                json=d,
                unlimited_gifts=d.get("unlimited_gifts"),
                limited_gifts=d.get("limited_gifts"),
                unique_gifts=d.get("unique_gifts"),
                premium_subscription=d.get("premium_subscription"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram

from typing import Union
from tgram.utils import convert_input_media


class SetBusinessAccountProfilePhoto:
    async def set_business_account_profile_photo(
        self: "tgram.TgBot",
        business_connection_id: str,
        photo: Union[
            "tgram.types.InputProfilePhotoStatic",
            "tgram.types.InputProfilePhotoAnimated",
        ],
        is_public: bool = None,
    ) -> bool:
        """
        Changes the profile photo of a managed business account.
        Requires the can_edit_profile_photo business bot right.

        Telegram documentation: https://core.telegram.org/bots/api#setbusinessaccountprofilephoto

        :param business_connection_id: Unique identifier of the business connection
        :type business_connection_id: :obj:`str`

        :param photo: The new profile photo to set
        :type photo: :class:`tgram.types.InputProfilePhotoStatic` or :class:`tgram.types.InputProfilePhotoAnimated`

        :param is_public: Pass True to set the public photo, which will be visible even if the main photo is hidden by the business account's privacy settings. An account can have only one public photo.
        :type is_public: :obj:`bool`, optional

        :return: On success, returns True.
        :rtype: :obj:`bool`
        """
        converted, file = convert_input_media([photo])
        result = await self(
            "setBusinessAccountProfilePhoto",
            business_connection_id=business_connection_id,
            photo=converted[0],
            is_public=is_public,
            **file,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import Optional


class Gift(Type_):
    """
    This object represents a gift that can be sent by the bot.

    Telegram Documentation: https://core.telegram.org/bots/api#gift

    :param id: Unique identifier of the gift.
    :type id: :obj:`int`

    :param sticker: The sticker that represents the gift
    :type sticker: :class:`tgram.types.Sticker`

    :param star_count: The number of Telegram Stars that must be paid to send the sticker
    :type star_count: :obj:`int`

    :param upgrade_star_count: Optional. The number of Telegram Stars that must be paid to upgrade the gift to a unique one
    :type upgrade_star_count: :obj:`int

    :param total_count: Optional. The total number of the gifts of this type that can be sent; for limited gifts only
    :type total_count: :obj:`int`

    :param remaining_count: Optional. The number of remaining gifts of this type that can be sent; for limited gifts only
    :type remaining_count: :obj:`int`

    :return: Instance of the class
    :rtype: :class:`tgram.types.Gift`
    """

    def __init__(
        self,
        id: "str" = None,
        sticker: "tgram.types.Sticker" = None,
        star_count: "int" = None,
        upgrade_star_count: "int" = None,
        total_count: "int" = None,
        remaining_count: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.id = id
        self.sticker = sticker
        self.star_count = star_count
        self.upgrade_star_count = upgrade_star_count
        self.total_count = total_count
        self.remaining_count = remaining_count

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.Gift"]:
        return (
            Gift(
                id=d.get("id"),
                sticker=tgram.types.Sticker._parse(me, d.get("sticker")),
                star_count=d.get("star_count"),
                upgrade_star_count=d.get("upgrade_star_count"),
                total_count=d.get("total_count"),
                remaining_count=d.get("remaining_count"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



from .base import StorageBase
from .kvsqlite_storage import KvsqliteStorage
from .redis_storage import RedisStorage

from . import utils

from tgram import sync

import inspect
import sys

for name, obj in inspect.getmembers(sys.modules[__name__]):
    if inspect.isclass(obj):
        sync.wrap(obj)

__all__ = ["StorageBase", "KvsqliteStorage", "RedisStorage", "utils"]



from tgram import utils
from json import dumps


# Base exception class that converts exception details to a JSON string
class StrException(Exception):
    def __init__(self, msg: str) -> None:
        super().__init__(msg)
        self.message = msg

    def __str__(self) -> str:
        return dumps(
            {
                "_": self.__class__.__name__,
                **{
                    k: v
                    for k, v in self.__dict__.items()
                    if not k.startswith("_") and v is not None
                },
            },
            indent=2,
            ensure_ascii=False,
        )


# Exception for chat not found error
class ChatNotFound(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for group chat migrated error
class GroupChatMigrated(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for invalid file ID error
class InvalidFileId(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for uneditable message error
class MessageUneditable(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for message not modified error
class MessageNotModified(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for empty message text error
class MessageTextEmpty(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for user not found error
class UserNotFound(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for wrong parameter error
class WrongParameter(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for termination by other long poll or webhook error
class TerminatedByOtherLongPollOrWebhook(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for bot blocked by user error
class BotWasBlocked(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for unable to send to bots error
class UnableSendToBots(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for bot kicked from group chat error
class BotWasKicked(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for user deactivated error
class UserDeactivated(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for flood wait error
class FloodWait(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])
        self.value: int = args[1].get("parameters", {}).get("retry_after", 0)


# Exception for unauthorized error
class Unauthorized(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Exception for webhook active error
class WebhookIsActive(StrException):
    def __init__(self, *args) -> None:
        super().__init__(args[0])


# Dictionary mapping error descriptions to exception classes and messages
exceptions = {
    "chat not found": (
        ChatNotFound,
        "The chat is unknown to the bot, Double check the provided chat_id.",
    ),
    "group chat was migrated to a supergroup chat": (
        GroupChatMigrated,
        "Occurs when a group chat has been converted/migrated to a supergroup, Check the provided chat_id and make sure the new Super Group ID is passed.",
    ),
    "invalid file id": (
        InvalidFileId,
        "The file id you are trying to retrieve doesn't exist.",
    ),
    "message can't be edited": (
        MessageUneditable,
        "The message text cannot be edited, Check is message belong to chat or is message exist in chat.",
    ),
    "message is not modified": (
        MessageNotModified,
        "The current and new message text and reply markups are the same, Actually change the text or reply markup of the message to be edited.",
    ),
    "message text is empty": (
        MessageTextEmpty,
        "The message text is empty or not provided, Provide a valid message text.",
    ),
    "user not found": (UserNotFound, "user_id is incorrect"),
    "wrong parameter action in request": (
        WrongParameter,
        "Occurs when the action property value is invalid, Provide a valid value to the action property as specified in the documentation.",
    ),
    "terminated by other long poll or webhook": (
        TerminatedByOtherLongPollOrWebhook,
        "You have already set up a webhook and are trying to get the updates via getUpdates.",
    ),
    "bot was blocked by the user": (BotWasBlocked, "The user has blocked the bot."),
    "bot can't send messages to bots": (
        UnableSendToBots,
        "You tried to send a message to another bot. This is not possible.",
    ),
    "bot was kicked from the group chat": (
        BotWasKicked,
        "Bot was kicked from the provided chat.",
    ),
    "user is deactivated": (
        UserDeactivated,
        "You're trying to perform an action on a user account that has been deactivated or deleted.",
    ),
    "Too Many Requests": (
        FloodWait,
        "You are hitting the API limit, more information here https://core.telegram.org/bots/faq#my-bot-is-hitting-limits-how-do-i-avoid-this",
    ),
    "Unauthorized": (
        Unauthorized,
        "Bot token is incorrect, Correct your bot token and try again.",
    ),
    "can't use getUpdates method while webhook is active; use deleteWebhook to delete the webhook first": (
        WebhookIsActive,
        "You are trying to use getUpdates while a webhook is active, Use deleteWebhook to delete the webhook first.",
    ),
}


# Exception for API errors
class APIException(StrException):
    def __init__(
        self, message: str, error_code: int, description: str, parameters: dict = None
    ) -> None:
        super().__init__(message)
        self.error_code = error_code
        self.description = description
        self.parameters = parameters

    @staticmethod
    def _from_json(json: dict):
        for i in exceptions:
            if i in json["description"]:
                e = exceptions.get(i)
                return e[0](e[1], json)

        return APIException(
            message=f"You got {json['error_code']} error: {json['description']}",
            error_code=json.get("error_code"),
            description=json.get("description"),
            parameters=utils.Json(json.get("parameters", {})),
        )


# Exception to stop propagation of an event
class StopPropagation(Exception):
    pass


class ContinuePropagation(Exception):
    pass


# Exception for muted errors
class MutedError(Exception):
    pass



import tgram
from typing import Union
from tgram.types import File
from pathlib import Path


class UploadStickerFile:
    async def upload_sticker_file(
        self: "tgram.TgBot",
        user_id: int,
        sticker: Union[Path, bytes, str],
        sticker_format: str,
    ) -> File:
        """
        Use this method to upload a .png file with a sticker for later use in createNewStickerSet and addStickerToSet
        methods (can be used multiple times). Returns the uploaded File on success.

        Telegram documentation: https://core.telegram.org/bots/api#uploadstickerfile

        :param user_id: User identifier of sticker set owner
        :type user_id: :obj:`int`

        :param png_sticker: DEPRECATED: PNG image with the sticker, must be up to 512 kilobytes in size, dimensions must not exceed 512px,
            and either width or height must be exactly 512px.
        :type png_sticker: :obj:`filelike object`

        :param sticker: A file with the sticker in .WEBP, .PNG, .TGS, or .WEBM format.
            See https://core.telegram.org/stickers for technical requirements. More information on Sending Files »
        :type sticker: :class:`tgram.types.InputFile`

        :param sticker_format: One of "static", "animated", "video".
        :type sticker_format: :obj:`str`

        :return: On success, the sent file is returned.
        :rtype: :class:`tgram.types.File`
        """

        result = await self(
            "uploadStickerFile",
            user_id=user_id,
            sticker=sticker,
            sticker_format=sticker_format,
        )
        return File._parse(me=self, d=result["result"])



import tgram
from typing import Union


class GetChatMemberCount:
    async def get_chat_member_count(
        self: "tgram.TgBot", chat_id: Union[int, str]
    ) -> int:
        """
        Use this method to get the number of members in a chat.

        Telegram documentation: https://core.telegram.org/bots/api#getchatmembercount

        :param chat_id: Unique identifier for the target chat or username of the target supergroup or channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :return: Number of members in the chat.
        :rtype: :obj:`int`
        """

        result = await self(
            "getChatMemberCount",
            chat_id=chat_id,
        )
        return result["result"]



import tgram
from typing import Union
from tgram.types import InlineKeyboardMarkup
from tgram.types import Poll

from tgram.utils import convert_to_inline_keyboard_markup


class StopPoll:
    async def stop_poll(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        message_id: int,
        business_connection_id: str = None,
        reply_markup: InlineKeyboardMarkup = None,
    ) -> Poll:
        """
        Use this method to stop a poll which was sent by the bot. On success, the stopped Poll is returned.

        Telegram documentation: https://core.telegram.org/bots/api#stoppoll

        :param chat_id: Unique identifier for the target chat or username of the target channel
        :type chat_id: :obj:`int` | :obj:`str`

        :param message_id: Identifier of the original message with the poll
        :type message_id: :obj:`int`

        :param reply_markup: A JSON-serialized object for a new message markup.
        :type reply_markup: :obj:`InlineKeyboardMarkup`

        :param business_connection_id: Identifier of the business connection to send the message through
        :type business_connection_id: :obj:`str`

        :return: On success, the stopped Poll is returned.
        :rtype: :obj:`tgram.types.Poll`
        """

        result = await self(
            "stopPoll",
            chat_id=chat_id,
            message_id=message_id,
            business_connection_id=business_connection_id,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
        )
        return Poll._parse(me=self, d=result["result"])



import aiohttp
import asyncio
import logging
import json
import tgram
import os
import io
import ssl
import certifi
import inspect

from ..methods import TelegramBotMethods
from ..decorators import Decorators
from ..errors import APIException
from ..utils import API_URL, get_file_name, ALL_UPDATES
from ..sync import wrap, async_to_sync
from ..storage import KvsqliteStorage, RedisStorage, StorageBase
from ..types.type_ import Type_, Response
from .dispatcher import Dispatcher
from concurrent.futures.thread import ThreadPoolExecutor

from typing import List, Any, Literal, Union, Optional
from collections import OrderedDict

from pathlib import Path
from importlib import import_module

from aiohttp.hdrs import USER_AGENT
from aiohttp.http import SERVER_SOFTWARE

logger = logging.getLogger(__name__)


class TgBot(TelegramBotMethods, Decorators, Dispatcher):
    """
    A class for creating a Telegram bot with extended functionality and
    support for various features like custom updates, storage engines, and more.

    Inherits from:
        - `TelegramBotMethods`: Provides various methods to interact with Telegram Bot API.
        - `Decorators`: Contains decorators to modify bot behavior.
        - `Dispatcher`: Manages event handling and dispatching for the bot.

    Attributes:
        bot_token (str): The bot token provided by the BotFather.
        api_url (str): The base API URL to communicate with Telegram servers.
        allowed_updates (List[str]): List of update types the bot should listen to.
        link_preview_options (tgram.types.LinkPreviewOptions): Options for link previews in messages.
        parse_mode (tgram.types.ParseMode): Default parse mode for formatting messages.
        protect_content (bool): Whether to protect content from being saved or forwarded.
        workers (int): Number of worker threads for handling updates.
        retry_after (Union[int, bool]): Time or condition for retrying failed requests.
        plugins (Union[Path, str]): Path to the directory containing bot plugins.
        skip_updates (bool): Whether to skip pending updates on bot startup.
        storage (Optional[StorageBase]): Storage engine instance for persisting data.
        storage_client (Any): Client for the storage engine (e.g., Redis or Kvsqlite).
        executor (ThreadPoolExecutor): Executor for running handler threads.
        loop (asyncio.AbstractEventLoop): Event loop used by the bot.
        is_running (bool): Indicates if the bot is currently running.
        me (tgram.types.User): The bot's user profile.
        handler_worker_tasks (List[asyncio.Task]): Tasks handling updates processing.
        locks_list (List[asyncio.Lock]): Locks used for synchronization.
        updates_queue (asyncio.Queue): Queue for managing incoming updates.
        groups (OrderedDict): Groups of handlers categorized by update types.
    """

    def __init__(
        self,
        bot_token: str,
        api_url: str = API_URL,
        allowed_updates: List[str] = [],
        link_preview_options: tgram.types.LinkPreviewOptions = None,
        parse_mode: tgram.types.ParseMode = None,
        protect_content: bool = None,
        workers: int = None,
        retry_after: Union[int, bool] = None,
        plugins: Union[Path, str] = None,
        skip_updates: bool = True,
        storage_engine: Union[
            KvsqliteStorage, RedisStorage, Literal["kvsqlite", "redis"]
        ] = None,
        storage_client: Any = None,
    ) -> None:
        self.bot_token = bot_token
        self.api_url = api_url
        self.allowed_updates = allowed_updates
        self.link_preview_options = link_preview_options
        self.parse_mode = parse_mode
        self.protect_content = protect_content
        self.workers = workers or min(32, (os.cpu_count() or 0) + 4)
        self.retry_after = retry_after
        self.plugins = Path(plugins) if isinstance(plugins, str) else plugins
        self.skip_updates = skip_updates
        self.storage: Optional[StorageBase] = None
        self.storage_client = storage_engine

        self.executor = ThreadPoolExecutor(self.workers, thread_name_prefix="Handlers")
        self.loop = asyncio.get_event_loop()

        self.is_running: bool = None
        self._me: "tgram.types.User" = None

        self._listen_handlers: List["tgram.types.Listener"] = []
        self._custom_types: dict = {}
        self._session: "aiohttp.ClientSession" = None

        self.handler_worker_tasks: List["asyncio.Task"] = []
        self.locks_list: List["asyncio.Lock"] = []
        self.updates_queue = asyncio.Queue()
        self.groups = OrderedDict()

        if not api_url.endswith("/"):
            api_url += "/"

        self._api_url: str = f"{api_url}bot{bot_token}/"

        # Initialize storage engine if provided
        if storage_engine:
            if isinstance(storage_engine, (KvsqliteStorage, RedisStorage)):
                self.storage = storage_engine
            else:
                if storage_engine.lower() == "kvsqlite":
                    try:
                        __import__("kvsqlite")
                    except ModuleNotFoundError:
                        raise ValueError(
                            "Please install kvsqlite module before using storage, see more https://pypi.org/project/Kvsqlite/"
                        )
                    else:
                        self.storage = KvsqliteStorage(self, storage_client)
                elif storage_engine.lower() == "redis":
                    try:
                        __import__("redis")
                    except ModuleNotFoundError:
                        raise ValueError(
                            "Please install redis module before using storage, see more https://pypi.org/project/redis/"
                        )
                    else:
                        self.storage = RedisStorage(self, storage_client)
                else:
                    raise ValueError(
                        "Unsupported storage engine {}, only {} are supported for now.".format(
                            storage_engine, " ,".join(i for i in ["redis", "kvsqlite"])
                        )
                    )

    def add_handler(self, handler: "tgram.handlers.Handler", group: int = 0) -> None:
        """
        Add a handler to the bot.

        Args:
            handler (tgram.handlers.Handler): The handler to add.
            group (int): The group to add the handler to.
        """
        if handler.type == "all":
            self.allowed_updates = ALL_UPDATES
        elif handler.type != "exception" and handler.type not in self.allowed_updates:
            self.allowed_updates.append(handler.type)

        self.loop.create_task(self._add_grouped_handler(handler, group))

    def remove_handler(self, handler: "tgram.handlers.Handler", group: int = 0) -> None:
        """
        Remove a handler from the bot.

        Args:
            handler (tgram.handlers.Handler): The handler to remove.
            group (int): The group to remove the handler from.
        """
        self.loop.create_task(self._remove_grouped_handler(handler, group))

    async def _new_session(self) -> None:
        """
        Create a new aiohttp session.
        """
        session = aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(
                limit=100,
                ssl_context=ssl.create_default_context(cafile=certifi.where()),
                ttl_dns_cache=3600,
            ),
            headers={USER_AGENT: f"{SERVER_SOFTWARE} tgram/{tgram.__version__}"},
        )
        self._session = session

    async def _get_session(self) -> "aiohttp.ClientSession":
        """
        Get the current aiohttp session, or create a new one if necessary.
        """
        if self._session is None or self._session.closed:
            await self._new_session()
        elif not self._session.loop.is_running():
            await self._session.close()
            await self._new_session()

        return self._session

    async def __call__(self, method: str, **kwargs) -> Response:
        """
        Make an API call to the Telegram Bot API.

        Args:
            method (str): The API method to call.
            **kwargs: Additional arguments for the API call.

        Returns:
            Any: The response from the API call.
        """
        request_url = self._api_url + method
        if method != "getUpdates":
            logger.info("Sending request using the method: %s", method)
        session = await self._get_session()
        data = aiohttp.FormData(quote_fields=False)
        has_files = False

        for key, value in kwargs.items():
            file = None
            if value is None or key == "timeout" or key == "retry":
                continue
            if isinstance(value, Path):
                has_files = True
                with open(value, "rb") as f:
                    value = f
                    file = f.read()
            elif isinstance(value, (io.BytesIO, io.BufferedReader, bytes)):
                has_files = True
                file = value if isinstance(value, bytes) else value.read()
            elif isinstance(value, (Type_, list)):
                value = json.dumps(value, ensure_ascii=False, default=Type_.default)
            else:
                value = str(value)
            data.add_field(
                key,
                file or value,
                filename=get_file_name(value) if file else None,
            )

        try:
            response = await session.request(
                "POST" if has_files else "GET",
                request_url,
                data=data,
                timeout=aiohttp.ClientTimeout(
                    total=kwargs.get("timeout", 60 if not has_files else 300)
                ),
            )
        except aiohttp.ClientConnectorError:
            logger.warning(
                "Network connection error occurred. Retrying in 5 seconds..."
            )
            await asyncio.sleep(5)
            return await self(method, **kwargs)

        if not self.is_running:
            await session.close()

        response_json = await response.json()

        if not response_json["ok"]:
            error = APIException._from_json(response_json)
            self.updates_queue.put_nowait({"e": error, "m": method, "kwargs": kwargs})
            try:
                raise error
            except tgram.errors.FloodWait as f:
                if self.retry_after and (not kwargs.get("retry")):
                    retry_after = (
                        f.value
                        if self.retry_after is True
                        else (
                            f.value if f.value < self.retry_after else self.retry_after
                        )
                    )
                    logger.warning(
                        "You got FloodWait for %s seconds, I will retry after %s",
                        f.value,
                        retry_after,
                    )
                    await asyncio.sleep(retry_after)
                    return await self(method, {"retry": 1, **kwargs})
            except Exception:
                raise

        return response_json

    def load_plugins(self) -> None:
        """
        Load plugins from the specified plugins directory.
        """
        for path in sorted(self.plugins.rglob("*.py")):
            module_path = ".".join(path.parent.parts + (path.stem,))
            module = import_module(module_path)
            for name in vars(module).keys():
                obj = getattr(module, name)

                if hasattr(obj, "handlers"):
                    for handler, group in obj.handlers:
                        if isinstance(handler, tgram.handlers.Handler) and isinstance(
                            group, int
                        ):
                            self.add_handler(handler, group)

    def customize(self, old: type, new: type) -> Literal[True]:
        """
        Customize a type used by the bot.

        Args:
            old (type): The old type to replace.
            new (type): The new type to use.

        Returns:
            Literal[True]: Always returns True if customization is successful.
        """
        if Type_ not in inspect.getmro(old):
            raise ValueError("You can't customize this type, it's not tgram type.")

        wrap(new)

        self._custom_types.update({old.__name__: new})

        return True

    @property
    def me(self) -> "tgram.types.User":
        """
        Get the bot's user profile.

        Returns:
            tgram.types.User: The bot's user profile.
        """
        if self._me:
            return self._me

        from urllib.request import urlopen

        response = urlopen(self._api_url + "getMe", timeout=60.0)

        response_json = json.loads(response.read().decode("utf-8"))

        if not response_json["ok"]:
            raise APIException._from_json(response)

        self._me = tgram.types.User._parse(self, response_json["result"])

        return self._me


wrap(TelegramBotMethods)
async_to_sync(TgBot, "__call__")



import tgram
from typing import List
from typing import Union
from tgram.types import ForceReply
from tgram.types import InlineKeyboardMarkup
from tgram.types import InputPollOption
from tgram.types import Message
from tgram.types import MessageEntity
from tgram.types import ReplyKeyboardMarkup
from tgram.types import ReplyKeyboardRemove
from tgram.types import ReplyParameters, ParseMode

from tgram.utils import convert_to_inline_keyboard_markup


class SendPoll:
    async def send_poll(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        question: str,
        options: List[InputPollOption],
        business_connection_id: str = None,
        message_thread_id: int = None,
        question_parse_mode: ParseMode = None,
        question_entities: List[MessageEntity] = None,
        is_anonymous: bool = None,
        type: str = None,
        allows_multiple_answers: bool = None,
        correct_option_id: int = None,
        explanation: str = None,
        explanation_parse_mode: ParseMode = None,
        explanation_entities: List[MessageEntity] = None,
        open_period: int = None,
        close_date: int = None,
        is_closed: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: Union[
            InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply
        ] = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        Use this method to send a native poll.
        On success, the sent Message is returned.

        Telegram documentation: https://core.telegram.org/bots/api#sendpoll

        :param chat_id: Unique identifier for the target chat or username of the target channel
        :type chat_id: :obj:`int` | :obj:`str`

        :param question: Poll question, 1-300 characters
        :type question: :obj:`str`

        :param options: A JSON-serialized list of 2-10 answer options
        :type options: :obj:`list` of :obj:`InputPollOption`

        :param is_anonymous: True, if the poll needs to be anonymous, defaults to True
        :type is_anonymous: :obj:`bool`

        :param type: Poll type, “quiz” or “regular”, defaults to “regular”
        :type type: :obj:`str`

        :param allows_multiple_answers: True, if the poll allows multiple answers, ignored for polls in quiz mode, defaults to False
        :type allows_multiple_answers: :obj:`bool`

        :param correct_option_id: 0-based identifier of the correct answer option. Available only for polls in quiz mode,
            defaults to None
        :type correct_option_id: :obj:`int`

        :param explanation: Text that is shown when a user chooses an incorrect answer or taps on the lamp icon in a quiz-style poll,
            0-200 characters with at most 2 line feeds after entities parsing
        :type explanation: :obj:`str`

        :param explanation_parse_mode: Mode for parsing entities in the explanation. See formatting options for more details.
        :type explanation_parse_mode: :obj:`str`

        :param open_period: Amount of time in seconds the poll will be active after creation, 5-600. Can't be used together with close_date.
        :type open_period: :obj:`int`

        :param close_date: Point in time (Unix timestamp) when the poll will be automatically closed.
        :type close_date: :obj:`int` | :obj:`datetime`

        :param is_closed: Pass True, if the poll needs to be immediately closed. This can be useful for poll preview.
        :type is_closed: :obj:`bool`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the poll allows multiple options to be voted simultaneously.
        :type allow_sending_without_reply: :obj:`bool`

        :param reply_markup: Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard,
            instructions to remove reply keyboard or to force a reply from the user.
        :type reply_markup: :obj:`InlineKeyboardMarkup` | :obj:`ReplyKeyboardMarkup` | :obj:`ReplyKeyboardRemove` | :obj:`ForceReply`

        :param timeout: Timeout in seconds for waiting for a response from the user.
        :type timeout: :obj:`int`

        :param explanation_entities: A JSON-serialized list of special entities that appear in the explanation,
            which can be specified instead of parse_mode
        :type explanation_entities: :obj:`list` of :obj:`MessageEntity`

        :param protect_content: Protects the contents of the sent message from forwarding and saving
        :type protect_content: :obj:`bool`

        :param message_thread_id: The identifier of a message thread, in which the poll will be sent
        :type message_thread_id: :obj:`int`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param business_connection_id: Identifier of the business connection to send the message through
        :type business_connection_id: :obj:`str`

        :param question_parse_mode: Mode for parsing entities in the question. See formatting options for more details. Currently, only custom emoji entities are allowed
        :type question_parse_mode: :obj:`str`

        :param question_entities: A JSON-serialized list of special entities that appear in the poll question. It can be specified instead of question_parse_mode
        :type question_entities: :obj:`list` of :obj:`MessageEntity`

        :param message_effect_id: Identifier of the message effect to apply to the sent message
        :type message_effect_id: :obj:`str`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :obj:`tgram.types.Message`
        """

        result = await self(
            "sendPoll",
            chat_id=chat_id,
            question=question,
            options=options,
            business_connection_id=business_connection_id,
            message_thread_id=message_thread_id,
            question_parse_mode=question_parse_mode,
            question_entities=question_entities,
            is_anonymous=is_anonymous,
            type=type,
            allows_multiple_answers=allows_multiple_answers,
            correct_option_id=correct_option_id,
            explanation=explanation,
            explanation_parse_mode=explanation_parse_mode,
            explanation_entities=explanation_entities,
            open_period=open_period,
            close_date=close_date,
            is_closed=is_closed,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
import io
from typing import Union
from pathlib import Path
from tgram.errors import APIException


class DownloadFile:
    async def download_file(
        self: "tgram.TgBot", file_id: str, file_path: str = None, in_memory: bool = None
    ) -> Union[Path, io.BytesIO]:
        file = await self.get_file(file_id)
        file_path = file_path or file.file_path.split("/")[1]
        url = self.api_url + f"file/bot{self.bot_token}/{file.file_path}"
        session = await self._get_session()
        async with session.request("GET", url=url) as response:
            if response.status != 200:
                raise APIException("Download file", response)
            result = await response.read()
        if in_memory:
            memory_file = io.BytesIO()
            memory_file.write(result)
            memory_file.seek(0)
            memory_file.name = file_path
            return memory_file
        else:
            with open(Path(file_path), "wb") as f:
                f.write(result)
            return Path(file_path)



import tgram

from tgram.types import PreparedInlineMessage, InlineQueryResult


class SavePreparedInlineMessage:
    async def save_prepared_inline_message(
        self: "tgram.TgBot",
        user_id: int,
        result: InlineQueryResult,
        allow_user_chats: bool = None,
        allow_bot_chats: bool = None,
        allow_group_chats: bool = None,
        allow_channel_chats: bool = None,
    ) -> PreparedInlineMessage:
        """
        Stores a message that can be sent by a user of a Mini App. Returns a PreparedInlineMessage object.

        Telegram documentation: https://core.telegram.org/bots/api#savepreparedinlinemessage
        """
        result = await self(
            "savePreparedInlineMessage",
            user_id=user_id,
            result=result,
            allow_user_chats=allow_user_chats,
            allow_bot_chats=allow_bot_chats,
            allow_group_chats=allow_group_chats,
            allow_channel_chats=allow_channel_chats,
        )

        return PreparedInlineMessage._parse(self, result["result"])



import tgram
from .type_ import Type_

from typing import List, Optional


class StarTransactions(Type_):
    """
    Contains a list of Telegram Star transactions.

    Telegram documentation: https://core.telegram.org/bots/api#startransactions

    :param transactions: The list of transactions
    :type transactions: :obj:`list` of :class:`StarTransaction`

    :return: Instance of the class
    :rtype: :class:`StarTransactions`

    """

    def __init__(
        self,
        transactions: List["tgram.types.StarTransaction"] = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.transactions = transactions

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.StarTransactions"]:
        return (
            StarTransactions(
                me=me,
                json=d,
                transactions=[
                    tgram.types.StarTransaction._parse(me=me, d=i)
                    for i in d.get("transactions")
                ]
                if d.get("transactions")
                else None,
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram


class SetBusinessAccountName:
    async def set_business_account_name(
        self: "tgram.TgBot",
        business_connection_id: str,
        first_name: str,
        last_name: str = None,
    ) -> bool:
        """
        Changes the first and last name of a managed business account.
        Requires the can_change_name business bot right.

        Telegram documentation: https://core.telegram.org/bots/api#setbusinessaccountname

        :param business_connection_id: Unique identifier of the business connection
        :type business_connection_id: :obj:`str`

        :param first_name: The new value of the first name for the business account; 1-64 characters
        :type first_name: :obj:`str`

        :param last_name: The new value of the last name for the business account; 0-64 characters
        :type last_name: :obj:`str`, optional

        :return: On success, returns True.
        :rtype: :obj:`bool`
        """

        result = await self(
            "setBusinessAccountName",
            business_connection_id=business_connection_id,
            first_name=first_name,
            last_name=last_name,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import List, Union, Optional


class InputSticker(Type_):
    """
    This object describes a sticker to be added to a sticker set.

    :param sticker: The added sticker. Pass a file_id as a String to send a file that already exists on the Telegram servers,
        pass an HTTP URL as a String for Telegram to get a file from the Internet, or upload a new one using multipart/form-data.
        Animated and video stickers can't be uploaded via HTTP URL.
    :type sticker: :obj:`str` or :obj:`tgram.types.InputFile`

    :param emoji_list: One or more(up to 20) emoji(s) corresponding to the sticker
    :type emoji_list: :obj:`list` of :obj:`str`

    :param mask_position: Optional. Position where the mask should be placed on faces. For “mask” stickers only.
    :type mask_position: :class:`tgram.types.MaskPosition`

    :param keywords: Optional. List of 0-20 search keywords for the sticker with total length of up to 64 characters.
        For “regular” and “custom_emoji” stickers only.
    :type keywords: :obj:`list` of :obj:`str`

    :param format: 	Format of the added sticker, must be one of “static” for a .WEBP or .PNG image, “animated” for a .TGS animation, “video” for a WEBM video
    :type format: :obj:`str`

    :return: Instance of the class
    :rtype: :class:`tgram.types.InputSticker`
    """

    def __init__(
        self,
        sticker: Union["tgram.types.InputFile", "str"] = None,
        format: "str" = None,
        emoji_list: List["str"] = None,
        mask_position: "tgram.types.MaskPosition" = None,
        keywords: List["str"] = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.sticker = sticker
        self.format = format
        self.emoji_list = emoji_list
        self.mask_position = mask_position
        self.keywords = keywords

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.InputSticker"]:
        return (
            InputSticker(
                me=me,
                json=d,
                sticker=d.get("sticker"),
                format=d.get("format"),
                emoji_list=d.get("emoji_list"),
                mask_position=tgram.types.MaskPosition._parse(
                    me=me, d=d.get("mask_position")
                ),
                keywords=d.get("keywords"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import List, Union, Optional


class CallbackB:
    async def answer(
        self: "tgram.types.CallbackQuery",
        text: str = None,
        show_alert: bool = None,
        url: str = None,
        cache_time: int = None,
    ) -> bool:
        """
        Answer the callback query.

        Args:
            text (str, optional): Notification text.
            show_alert (bool, optional): If True, an alert will be shown.
            url (str, optional): URL to be opened.
            cache_time (int, optional): The maximum amount of time in seconds that the result of the callback query may be cached client-side.

        Returns:
            bool: True on success.
        """
        return await self._me.answer_callback_query(
            self.id,
            text=text,
            show_alert=show_alert,
            url=url,
            cache_time=cache_time,
        )

    async def edit_message_text(
        self: "tgram.types.CallbackQuery",
        text: str,
        parse_mode: "tgram.types.ParseMode" = None,
        entities: List["tgram.types.MessageEntity"] = None,
        link_preview_options: "tgram.types.LinkPreviewOptions" = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edit the text of the message.

        Args:
            text (str): New text of the message.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the message text.
            entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in message text.
            link_preview_options (tgram.types.LinkPreviewOptions, optional): Options for link previews.
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): New inline keyboard.

        Returns:
            Union[tgram.types.Message, bool]: The edited message or True on success.
        """
        return await self._me.edit_message_text(
            text=text,
            chat_id=self.message.chat.id
            if (self.message and self.message.chat)
            else None,
            message_id=self.message.id if self.message else None,
            inline_message_id=self.inline_message_id,
            parse_mode=parse_mode,
            entities=entities,
            link_preview_options=link_preview_options,
            reply_markup=reply_markup,
        )

    async def edit_message_caption(
        self: "tgram.types.CallbackQuery",
        caption: str = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        show_caption_above_media: bool = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edit the caption of the message.

        Args:
            caption (str, optional): New caption of the message.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the message caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in message caption.
            show_caption_above_media (bool, optional): If True, the caption will be shown above the media.
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): New inline keyboard.

        Returns:
            Union[tgram.types.Message, bool]: The edited message or True on success.
        """
        return await self._me.edit_message_caption(
            chat_id=self.message.chat.id
            if (self.message and self.message.chat)
            else None,
            message_id=self.message.id if self.message else None,
            inline_message_id=self.inline_message_id,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            reply_markup=reply_markup,
        )

    async def edit_message_reply_markup(
        self: "tgram.types.CallbackQuery",
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edit the reply markup of the message.

        Args:
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): New inline keyboard.

        Returns:
            Union[tgram.types.Message, bool]: The edited message or True on success.
        """
        return await self._me.edit_message_reply_markup(
            chat_id=self.message.chat.id
            if (self.message and self.message.chat)
            else None,
            message_id=self.message.id if self.message else None,
            inline_message_id=self.inline_message_id,
            reply_markup=reply_markup,
        )

    async def edit_message_media(
        self: "tgram.types.CallbackQuery",
        media: "tgram.types.InputMedia",
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edit the media of the message.

        Args:
            media (tgram.types.InputMedia): New media of the message.
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): New inline keyboard.

        Returns:
            Union[tgram.types.Message, bool]: The edited message or True on success.
        """
        return await self._me.edit_message_media(
            media=media,
            inline_message_id=self.inline_message_id,
            chat_id=self.message.chat.id
            if (self.message and self.message.chat)
            else None,
            message_id=self.message.id if self.message else None,
            reply_markup=reply_markup,
        )

    async def edit_message_live_location(
        self: "tgram.types.CallbackQuery",
        latitude: float,
        longitude: float,
        live_period: int,
        horizontal_accuracy: float = None,
        heading: int = None,
        proximity_alert_radius: int = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edit the live location of the message.

        Args:
            latitude (float): Latitude of the new location.
            longitude (float): Longitude of the new location.
            live_period (int): Period in seconds for which the location will be updated.

        Returns:
            Union[tgram.types.Message, bool]: The edited message or True on success.
        """
        return await self._me.edit_message_live_location(
            latitude=latitude,
            longitude=longitude,
            chat_id=self.message.chat.id
            if (self.message and self.message.chat)
            else None,
            message_id=self.message.id if self.message else None,
            inline_message_id=self.inline_message_id,
            live_period=live_period,
            horizontal_accuracy=horizontal_accuracy,
            heading=heading,
            proximity_alert_radius=proximity_alert_radius,
            reply_markup=reply_markup,
        )

    @property
    def user(self: "tgram.types.CallbackQuery") -> Optional["tgram.types.User"]:
        """
        Get the user who sent the callback query.

        Returns:
            Optional[tgram.types.User]: The user who sent the callback query.
        """
        return self.from_user

    sender_user = user



import tgram


class ConvertGiftToStars:
    async def convert_gift_to_stars(
        self: "tgram.TgBot",
        business_connection_id: str,
        owned_gift_id: str,
    ) -> bool:
        """
        Converts a given regular gift to Telegram Stars. Requires the can_convert_gifts_to_stars business bot right. Returns True on success.

        Parameters:
            business_connection_id (str): Unique identifier of the business connection.
            owned_gift_id (str): Unique identifier of the regular gift that should be converted to Telegram Stars.
        """
        result = await self(
            "convertGiftToStars",
            business_connection_id=business_connection_id,
            owned_gift_id=owned_gift_id,
        )
        return result["result"]



import tgram

from typing import Any, Coroutine, Literal, Optional


class Runner:
    async def run(self: "tgram.TgBot", main: Optional[Coroutine] = None) -> Any:
        """
        Use this method to run a couroutine function or handle new updates.

        :param main: The couroutine function.
        :type main: :obj:`Coroutine`

        :rtype: :obj:`Any`
        """
        if main is None:
            return await self.run_for_updates()
        return await main

    async def stop(self) -> Literal[True]:
        """
        Use this method to stop getting and handling new updates.

        :rtype: :obj:`bool`
        """
        self.is_running = False
        return True



import tgram
from .type_ import Type_

from typing import Optional


class PaidMessagePriceChanged(Type_):
    """
    Describes a service message about a change in the price of paid messages within a chat.

    :param paid_message_star_count: The new number of Telegram Stars that must be paid by non-administrator users of the supergroup chat for each sent message
    :type paid_message_star_count: :obj:`int`

    :return: Instance of the class
    :rtype: :class:`tgram.types.PaidMessagePriceChanged`
    """

    def __init__(
        self,
        paid_message_star_count: int = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.paid_message_star_count = paid_message_star_count

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["PaidMessagePriceChanged"]:
        return (
            PaidMessagePriceChanged(
                me=me,
                json=d,
                paid_message_star_count=d.get("paid_message_star_count"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import List
from tgram.types import PassportElementError


class SetPassportDataErrors:
    async def set_passport_data_errors(
        self: "tgram.TgBot", user_id: int, errors: List[PassportElementError]
    ) -> bool:
        result = await self(
            "setPassportDataErrors",
            user_id=user_id,
            errors=errors,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import Optional


class SwitchInlineQueryChosenChat(Type_):
    """
    Represents an inline button that switches the current user to inline mode in a chosen chat,
    with an optional default inline query.

    Telegram Documentation: https://core.telegram.org/bots/api#inlinekeyboardbutton

    :param query: Optional. The default inline query to be inserted in the input field.
                  If left empty, only the bot's username will be inserted
    :type query: :obj:`str`

    :param allow_user_chats: Optional. True, if private chats with users can be chosen
    :type allow_user_chats: :obj:`bool`

    :param allow_bot_chats: Optional. True, if private chats with bots can be chosen
    :type allow_bot_chats: :obj:`bool`

    :param allow_group_chats: Optional. True, if group and supergroup chats can be chosen
    :type allow_group_chats: :obj:`bool`

    :param allow_channel_chats: Optional. True, if channel chats can be chosen
    :type allow_channel_chats: :obj:`bool`

    :return: Instance of the class
    :rtype: :class:`SwitchInlineQueryChosenChat`
    """

    def __init__(
        self,
        query: "str" = None,
        allow_user_chats: "bool" = None,
        allow_bot_chats: "bool" = None,
        allow_group_chats: "bool" = None,
        allow_channel_chats: "bool" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.query = query
        self.allow_user_chats = allow_user_chats
        self.allow_bot_chats = allow_bot_chats
        self.allow_group_chats = allow_group_chats
        self.allow_channel_chats = allow_channel_chats

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.SwitchInlineQueryChosenChat"]:
        return (
            SwitchInlineQueryChosenChat(
                me=me,
                json=d,
                query=d.get("query"),
                allow_user_chats=d.get("allow_user_chats"),
                allow_bot_chats=d.get("allow_bot_chats"),
                allow_group_chats=d.get("allow_group_chats"),
                allow_channel_chats=d.get("allow_channel_chats"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
import inspect


class AsyncProperty:
    def __init__(self, f, bot: "tgram.TgBot") -> None:
        self.f = f
        self.bot = bot

    async def __call__(self, *args):
        if inspect.iscoroutinefunction(self.f):
            r = await self.f()
        else:
            r = await self.bot.loop.run_in_executor(self.bot.executor, self.f, *args)

        return r



import tgram


class SetUserEmojiStatus:
    async def set_user_emoji_status(
        self: "tgram.Tgbot",
        user_id: int,
        emoji_status_custom_emoji_id: str = None,
        emoji_status_expiration_date: int = None,
    ) -> bool:
        """
        Changes the emoji status for a given user that previously allowed the bot to manage their emoji status via the Mini App method requestEmojiStatusAccess. Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#setuseremojistatus

        :param user_id: OUnique identifier of the target user.
        :type user_id: :obj:`int`

        :param emoji_status_custom_emoji_id: Custom emoji identifier of the emoji status to set. Pass an empty string to remove the status.
        :type emoji_status_custom_emoji_id: :obj:`str`

        :param emoji_status_expiration_date: Expiration date of the emoji status, if any.
        :type emoji_status_expiration_date: :obj:`int`

        :return: True on success.
        """
        result = await self(
            "setUserEmojiStatus",
            user_id=user_id,
            emoji_status_custom_emoji_id=emoji_status_custom_emoji_id,
            emoji_status_expiration_date=emoji_status_expiration_date,
        )
        return result["result"]



import tgram
from typing import List
from tgram.types import GameHighScore


class GetGameHighScores:
    async def get_game_high_scores(
        self: "tgram.TgBot",
        user_id: int,
        chat_id: int = None,
        message_id: int = None,
        inline_message_id: str = None,
    ) -> List[GameHighScore]:
        """
        Use this method to get data for high score tables. Will return the score of the specified user and several of
        their neighbors in a game. On success, returns an Array of GameHighScore objects.

        This method will currently return scores for the target user, plus two of their closest neighbors on each side.
        Will also return the top three users if the user and their neighbors are not among them.
        Please note that this behavior is subject to change.

        Telegram documentation: https://core.telegram.org/bots/api#getgamehighscores

        :param user_id: User identifier
        :type user_id: :obj:`int`

        :param chat_id: Required if inline_message_id is not specified. Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param message_id: Required if inline_message_id is not specified. Identifier of the sent message
        :type message_id: :obj:`int`

        :param inline_message_id: Required if chat_id and message_id are not specified. Identifier of the inline message
        :type inline_message_id: :obj:`str`

        :return: On success, returns an Array of GameHighScore objects.
        :rtype: List[types.GameHighScore]
        """

        result = await self(
            "getGameHighScores",
            user_id=user_id,
            chat_id=chat_id,
            message_id=message_id,
            inline_message_id=inline_message_id,
        )
        return [GameHighScore._parse(me=self, d=i) for i in result["result"]]



import tgram


class SetCustomEmojiStickerSetThumbnail:
    async def set_custom_emoji_sticker_set_thumbnail(
        self: "tgram.TgBot", name: str, custom_emoji_id: str = None
    ) -> bool:
        """
        Use this method to set the thumbnail of a custom emoji sticker set.
        Returns True on success.

        :param name: Sticker set name
        :type name: :obj:`str`

        :param custom_emoji_id: Custom emoji identifier of a sticker from the sticker set; pass an empty string to drop the thumbnail and use the first sticker as the thumbnail.
        :type custom_emoji_id: :obj:`str`

        :return: Returns True on success.
        :rtype: :obj:`bool`
        """

        result = await self(
            "setCustomEmojiStickerSetThumbnail",
            name=name,
            custom_emoji_id=custom_emoji_id,
        )
        return result["result"]



import tgram
from typing import Union
from tgram.types import ChatInviteLink


class CreateChatInviteLink:
    async def create_chat_invite_link(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        name: str = None,
        expire_date: int = None,
        member_limit: int = None,
        creates_join_request: bool = None,
    ) -> ChatInviteLink:
        """
        Use this method to create an additional invite link for a chat. The bot must be an administrator in the chat for this to work and
        must have the appropriate administrator rights.
        The link can be revoked using the method revokeChatInviteLink.
        Returns the new invite link as ChatInviteLink object.

        Telegram documentation: https://core.telegram.org/bots/api#createchatinvitelink

        :param chat_id: Id: Unique identifier for the target chat or username of the target channel
            (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param name: Invite link name; 0-32 characters
        :type name: :obj:`str`

        :param expire_date: Point in time (Unix timestamp) when the link will expire
        :type expire_date: :obj:`int`

        :param member_limit: Maximum number of users that can be members of the chat simultaneously
        :type member_limit: :obj:`int`

        :param creates_join_request: True, if users joining the chat via the link need to be approved by chat administrators. If True, member_limit can't be specified
        :type creates_join_request: :obj:`bool`

        :return: Returns the new invite link as ChatInviteLink object.
        :rtype: :class:`tgram.types.ChatInviteLink`
        """

        result = await self(
            "createChatInviteLink",
            chat_id=chat_id,
            name=name,
            expire_date=expire_date,
            member_limit=member_limit,
            creates_join_request=creates_join_request,
        )
        return ChatInviteLink._parse(me=self, d=result["result"])



import tgram
import os
import re

from typing import Optional, List, Union, Tuple
from pathlib import Path
from io import BytesIO

from tgram import utils


def message_origin_parse(
    d: Optional[dict] = None, me: Optional["tgram.TgBot"] = None
) -> Optional["tgram.types.MessageOrigin"]:
    if d is None:
        return None

    origin_type = d["type"]

    return (
        tgram.types.MessageOriginUser._parse(me=me, d=d)
        if origin_type == "user"
        else tgram.types.MessageOriginHiddenUser._parse(me=me, d=d)
        if origin_type == "hidden_user"
        else tgram.types.MessageOriginChat._parse(me=me, d=d)
        if origin_type == "chat"
        else tgram.types.MessageOriginChannel._parse(me=me, d=d)
    )


def convert_input_media(
    x: List[
        Union[
            "tgram.types.InputMedia",
            "tgram.types.InputPaidMedia",
            "tgram.types.InputProfilePhotoAnimated",
            "tgram.types.InputProfilePhotoStatic",
        ]
    ],
):
    files = {}
    count = 1

    def attach_file(obj, attr):
        nonlocal count
        media = getattr(obj, attr, None)
        if media is None:
            return
        is_path = isinstance(media, (Path, str)) and os.path.isfile(media)
        is_bytes = isinstance(media, (bytes, BytesIO))
        if is_path or is_bytes:
            files[f"file_{count}"] = utils.get_file_path(media)
            setattr(obj, attr, f"attach://file_{count}")
            count += 1

    for y in x:
        # Handle profile photos
        if isinstance(
            y,
            (
                tgram.types.InputProfilePhotoAnimated,
                tgram.types.InputProfilePhotoStatic,
            ),
        ):
            attr_name = "animation" if hasattr(y, "animation") else "photo"
            attach_file(y, attr_name)
        else:
            # Handle main media
            attach_file(y, "media")
            # Optionally handle thumbnail and cover
            if hasattr(y, "thumbnail") and getattr(y, "thumbnail", None):
                attach_file(y, "thumbnail")
            if hasattr(y, "cover") and getattr(y, "cover", None):
                attach_file(y, "cover")

    return x, files


def reaction_type_parse(
    bot: "tgram.TgBot",
    x: Optional[Union[List[dict], dict]],
) -> "tgram.types.ReactionType":
    if x is None:
        return None

    x = x if isinstance(x, list) else [x]

    return [
        (
            tgram.types.ReactionTypeCustomEmoji._parse(bot, i)
            if i["type"] == "custom_emoji"
            else tgram.types.ReactionTypeEmoji._parse(bot, i)
            if i["type"] == "emoji"
            else tgram.types.ReactionTypePaid._parse(bot, i)
        )
        for i in x
    ]


pattern = re.compile(
    r"^(https?):\/\/" r"([a-zA-Z0-9.-]+)" r"(\.[a-zA-Z]{2,})" r"(\/[^\s]*)?$"
)


def convert_to_inline_keyboard_markup(v: List[List[Tuple]]):
    return tgram.types.InlineKeyboardMarkup(
        [
            [
                tgram.types.InlineKeyboardButton(
                    x,
                    callback_data=y if not re.match(pattern, y) else None,
                    url=y if re.match(pattern, y) else None,
                    user_id=y if isinstance(y, int) else None,
                )
                for x, y in z
            ]
            for z in v
        ]
    )



import tgram
from .type_ import Type_

from typing import Optional, Union

from pathlib import Path


class InputPaidMediaPhoto(Type_):
    """
    The paid media to send is a photo.

    Telegram documentation: https://core.telegram.org/bots/api#inputpaidmediaphoto

    :param type: Type of the media, must be photo
    :type type: :obj:`str`

    :param media: File to send. Pass a file_id to send a file that exists on the Telegram servers (recommended), pass an HTTP URL for
        Telegram to get a file from the Internet, or pass “attach://<file_attach_name>” to upload a new one using multipart/form-data
        under <file_attach_name> name. More information on Sending Files »
    :type media: :obj:`str`

    :return: Instance of the class
    :rtype: :class:`InputPaidMediaPhoto`
    """

    def __init__(
        self,
        media: Union["Path", "str"] = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.type = "photo"
        self.media = media

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.InputPaidMediaPhoto"]:
        return (
            InputPaidMediaPhoto(me=me, json=d, type=d.get("type"), media=d.get("media"))
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from .type_ import Type_

from typing import List, Optional


class Gifts(Type_):
    """
    This object represent a list of gifts.

    Telegram Documentation: https://core.telegram.org/bots/api#gifts

    :param gifts: Photo that will be displayed in the game message in chats.
    :type gifts: :obj:`list` of :class:`tgram.types.Gift`

    :return: Instance of the class
    :rtype: :class:`tgram.types.Gifts`
    """

    def __init__(
        self,
        gifts: List["tgram.types.Gift"] = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.gifts = gifts

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.Gifts"]:
        return (
            Gifts(gifts=[tgram.types.Gift._parse(me, i) for i in d.get("gifts")])
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



from .client import TgBot
from .dispatcher import Dispatcher

from tgram import sync

import inspect
import sys

for name, obj in inspect.getmembers(sys.modules[__name__]):
    if inspect.isclass(obj):
        sync.wrap(obj)

__all__ = ["TgBot", "Dispatcher"]



import tgram
from tgram.types import InlineQueryResult
from tgram.types import SentWebAppMessage


class AnswerWebAppQuery:
    async def answer_web_app_query(
        self: "tgram.TgBot", web_app_query_id: str, result: InlineQueryResult
    ) -> SentWebAppMessage:
        """
        Use this method to set the result of an interaction with a Web App and
        send a corresponding message on behalf of the user to the chat from which
        the query originated.
        On success, a SentWebAppMessage object is returned.

        Telegram Documentation: https://core.telegram.org/bots/api#answerwebappquery

        :param web_app_query_id: Unique identifier for the query to be answered
        :type web_app_query_id: :obj:`str`

        :param result: A JSON-serialized object describing the message to be sent
        :type result: :class:`tgram.types.InlineQueryResultBase`

        :return: On success, a SentWebAppMessage object is returned.
        :rtype: :class:`tgram.types.SentWebAppMessage`
        """

        result = await self(
            "answerWebAppQuery",
            web_app_query_id=web_app_query_id,
            result=result,
        )
        return SentWebAppMessage._parse(me=self, d=result["result"])



import tgram


class SetStickerSetTitle:
    async def set_sticker_set_title(self: "tgram.TgBot", name: str, title: str) -> bool:
        """
        Use this method to set the title of a created sticker set.
        Returns True on success.

        :param name: Sticker set name
        :type name: :obj:`str`

        :param title: New sticker set title
        :type title: :obj:`str`

        :return: Returns True on success.
        :rtype: :obj:`bool`
        """

        result = await self(
            "setStickerSetTitle",
            name=name,
            title=title,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import Optional


class Location(Type_):
    """
    This object represents a point on the map.

    Telegram Documentation: https://core.telegram.org/bots/api#location

    :param longitude: Longitude as defined by sender
    :type longitude: :obj:`float`

    :param latitude: Latitude as defined by sender
    :type latitude: :obj:`float`

    :param horizontal_accuracy: Optional. The radius of uncertainty for the location, measured in meters; 0-1500
    :type horizontal_accuracy: :obj:`float` number

    :param live_period: Optional. Time relative to the message sending date, during which the location can be updated;
        in seconds. For active live locations only.
    :type live_period: :obj:`int`

    :param heading: Optional. The direction in which user is moving, in degrees; 1-360. For active live locations only.
    :type heading: :obj:`int`

    :param proximity_alert_radius: Optional. The maximum distance for proximity alerts about approaching another
        chat member, in meters. For sent live locations only.
    :type proximity_alert_radius: :obj:`int`

    :return: Instance of the class
    :rtype: :class:`tgram.types.Location`
    """

    def __init__(
        self,
        latitude: "float" = None,
        longitude: "float" = None,
        horizontal_accuracy: "float" = None,
        live_period: "int" = None,
        heading: "int" = None,
        proximity_alert_radius: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.latitude = latitude
        self.longitude = longitude
        self.horizontal_accuracy = horizontal_accuracy
        self.live_period = live_period
        self.heading = heading
        self.proximity_alert_radius = proximity_alert_radius

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.Location"]:
        return (
            Location(
                me=me,
                json=d,
                latitude=d.get("latitude"),
                longitude=d.get("longitude"),
                horizontal_accuracy=d.get("horizontal_accuracy"),
                live_period=d.get("live_period"),
                heading=d.get("heading"),
                proximity_alert_radius=d.get("proximity_alert_radius"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from .type_ import Type_

from typing import List, Optional


class InputChecklist(Type_):
    """
    Describes a checklist to create.

    Telegram Documentation: https://core.telegram.org/bots/api#inputchecklist

    :param title: Title of the checklist; 1-255 characters after entities parsing
    :type title: :obj:`str`

    :param parse_mode: Optional. Mode for parsing entities in the title. See formatting options for more details.
    :type parse_mode: :obj:`str`

    :param title_entities: Optional. List of special entities that appear in the title, which can be specified instead of parse_mode.
        Currently, only bold, italic, underline, strikethrough, spoiler, and custom_emoji entities are allowed.
    :type title_entities: :obj:`list` of :class:`tgram.types.MessageEntity`

    :param tasks: List of 1-30 tasks in the checklist
    :type tasks: :obj:`list` of :class:`tgram.types.InputChecklistTask`

    :param others_can_add_tasks: Optional. Pass True if other users can add tasks to the checklist
    :type others_can_add_tasks: :obj:`bool`

    :param others_can_mark_tasks_as_done: Optional. Pass True if other users can mark tasks as done or not done in the checklist
    :type others_can_mark_tasks_as_done: :obj:`bool`
    """

    def __init__(
        self,
        title: str,
        tasks: List["tgram.types.InputChecklistTask"],
        parse_mode: "str" = None,
        title_entities: List["tgram.types.MessageEntity"] = None,
        others_can_add_tasks: "bool" = None,
        others_can_mark_tasks_as_done: "bool" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.title = title
        self.parse_mode = parse_mode
        self.title_entities = title_entities
        self.tasks = tasks
        self.others_can_add_tasks = others_can_add_tasks
        self.others_can_mark_tasks_as_done = others_can_mark_tasks_as_done

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.InputChecklist"]:
        return (
            InputChecklist(
                me=me,
                json=d,
                title=d.get("title"),
                parse_mode=d.get("parse_mode"),
                title_entities=[
                    tgram.types.MessageEntity._parse(me=me, d=i)
                    for i in d.get("title_entities")
                ]
                if d.get("title_entities")
                else None,
                tasks=[
                    tgram.types.InputChecklistTask._parse(me=me, d=i)
                    for i in d.get("tasks")
                ]
                if d.get("tasks")
                else None,
                others_can_add_tasks=d.get("others_can_add_tasks"),
                others_can_mark_tasks_as_done=d.get("others_can_mark_tasks_as_done"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from .type_ import Type_

from typing import Optional


class Contact(Type_):
    """
    This object represents a phone contact.

    Telegram Documentation: https://core.telegram.org/bots/api#contact

    :param phone_number: Contact's phone number
    :type phone_number: :obj:`str`

    :param first_name: Contact's first name
    :type first_name: :obj:`str`

    :param last_name: Optional. Contact's last name
    :type last_name: :obj:`str`

    :param user_id: Optional. Contact's user identifier in Telegram. This number may have more than 32 significant bits
        and some programming languages may have difficulty/silent defects in interpreting it. But it has at most 52
        significant bits, so a 64-bit integer or double-precision float type are safe for storing this identifier.
    :type user_id: :obj:`int`

    :param vcard: Optional. Additional data about the contact in the form of a vCard
    :type vcard: :obj:`str`

    :return: Instance of the class
    :rtype: :class:`tgram.types.Contact`
    """

    def __init__(
        self,
        phone_number: "str" = None,
        first_name: "str" = None,
        last_name: "str" = None,
        user_id: "int" = None,
        vcard: "str" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.phone_number = phone_number
        self.first_name = first_name
        self.last_name = last_name
        self.user_id = user_id
        self.vcard = vcard

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.Contact"]:
        return (
            Contact(
                me=me,
                json=d,
                phone_number=d.get("phone_number"),
                first_name=d.get("first_name"),
                last_name=d.get("last_name"),
                user_id=d.get("user_id"),
                vcard=d.get("vcard"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from .type_ import Type_
from typing import Optional


class UniqueGiftModel(Type_):
    """
    This object describes the model of a unique gift.

    Telegram Documentation: https://core.telegram.org/bots/api#uniquegiftmodel

    :param name: Name of the model
    :type name: :obj:`str`
    :param sticker: The sticker that represents the unique gift
    :type sticker: :class:`tgram.types.Sticker`
    :param rarity_per_mille: The number of unique gifts that receive this model for every 1000 gifts upgraded
    :type rarity_per_mille: :obj:`int`
    """

    def __init__(
        self,
        name: "str" = None,
        sticker: "tgram.types.Sticker" = None,
        rarity_per_mille: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.name = name
        self.sticker = sticker
        self.rarity_per_mille = rarity_per_mille

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["UniqueGiftModel"]:
        return (
            UniqueGiftModel(
                name=d.get("name"),
                sticker=tgram.types.Sticker._parse(me, d.get("sticker")),
                rarity_per_mille=d.get("rarity_per_mille"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )


class UniqueGiftSymbol(Type_):
    """
    This object describes the symbol shown on the pattern of a unique gift.

    :param name: Name of the symbol
    :type name: :obj:`str`
    :param sticker: The sticker that represents the unique gift
    :type sticker: :class:`tgram.types.Sticker`
    :param rarity_per_mille: The number of unique gifts that receive this model for every 1000 gifts upgraded
    :type rarity_per_mille: :obj:`int`
    """

    def __init__(
        self,
        name: "str" = None,
        sticker: "tgram.types.Sticker" = None,
        rarity_per_mille: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.name = name
        self.sticker = sticker
        self.rarity_per_mille = rarity_per_mille

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["UniqueGiftSymbol"]:
        return (
            UniqueGiftSymbol(
                name=d.get("name"),
                sticker=tgram.types.Sticker._parse(me, d.get("sticker")),
                rarity_per_mille=d.get("rarity_per_mille"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )


class UniqueGiftBackdropColors(Type_):
    """
    This object describes the colors of the backdrop of a unique gift.

    :param center_color: The color in the center of the backdrop in RGB format
    :type center_color: :obj:`int`
    :param edge_color: The color on the edges of the backdrop in RGB format
    :type edge_color: :obj:`int`
    :param symbol_color: The color to be applied to the symbol in RGB format
    :type symbol_color: :obj:`int`
    :param text_color: The color for the text on the backdrop in RGB format
    :type text_color: :obj:`int`
    """

    def __init__(
        self,
        center_color: "int" = None,
        edge_color: "int" = None,
        symbol_color: "int" = None,
        text_color: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.center_color = center_color
        self.edge_color = edge_color
        self.symbol_color = symbol_color
        self.text_color = text_color

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["UniqueGiftBackdropColors"]:
        return (
            UniqueGiftBackdropColors(
                center_color=d.get("center_color"),
                edge_color=d.get("edge_color"),
                symbol_color=d.get("symbol_color"),
                text_color=d.get("text_color"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )


class UniqueGiftBackdrop(Type_):
    """
    This object describes the backdrop of a unique gift.

    :param name: Name of the backdrop
    :type name: :obj:`str`
    :param colors: Colors of the backdrop
    :type colors: :class:`tgram.types.UniqueGiftBackdropColors`
    :param rarity_per_mille: The number of unique gifts that receive this backdrop for every 1000 gifts upgraded
    :type rarity_per_mille: :obj:`int`
    """

    def __init__(
        self,
        name: "str" = None,
        colors: "UniqueGiftBackdropColors" = None,
        rarity_per_mille: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.name = name
        self.colors = colors
        self.rarity_per_mille = rarity_per_mille

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["UniqueGiftBackdrop"]:
        return (
            UniqueGiftBackdrop(
                name=d.get("name"),
                colors=UniqueGiftBackdropColors._parse(me, d.get("colors")),
                rarity_per_mille=d.get("rarity_per_mille"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )


class UniqueGift(Type_):
    """
    This object describes a unique gift that was upgraded from a regular gift.

    :param base_name: Human-readable name of the regular gift from which this unique gift was upgraded
    :type base_name: :obj:`str`
    :param name: Unique name of the gift. This name can be used in https://t.me/nft/... links and story areas
    :type name: :obj:`str`
    :param number: Unique number of the upgraded gift among gifts upgraded from the same regular gift
    :type number: :obj:`int`
    :param model: Model of the gift
    :type model: :class:`tgram.types.UniqueGiftModel`
    :param symbol: Symbol of the gift
    :type symbol: :class:`tgram.types.UniqueGiftSymbol`
    :param backdrop: Backdrop of the gift
    :type backdrop: :class:`tgram.types.UniqueGiftBackdrop`
    """

    def __init__(
        self,
        base_name: "str" = None,
        name: "str" = None,
        number: "int" = None,
        model: "UniqueGiftModel" = None,
        symbol: "UniqueGiftSymbol" = None,
        backdrop: "UniqueGiftBackdrop" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.base_name = base_name
        self.name = name
        self.number = number
        self.model = model
        self.symbol = symbol
        self.backdrop = backdrop

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["UniqueGift"]:
        return (
            UniqueGift(
                base_name=d.get("base_name"),
                name=d.get("name"),
                number=d.get("number"),
                model=UniqueGiftModel._parse(me, d.get("model")),
                symbol=UniqueGiftSymbol._parse(me, d.get("symbol")),
                backdrop=UniqueGiftBackdrop._parse(me, d.get("backdrop")),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )


class UniqueGiftInfo(Type_):
    """
    Describes a service message about a unique gift that was sent or received.

    :param gift: Information about the gift
    :type gift: :class:`tgram.types.UniqueGift`
    :param origin: Origin of the gift. Currently, either “upgrade” for gifts upgraded from regular gifts, “transfer” for gifts transferred from other users or channels, or “resale” for gifts bought from other users
    :type origin: :obj:`str`
    :param owned_gift_id: Optional. Unique identifier of the received gift for the bot; only present for gifts received on behalf of business accounts
    :type owned_gift_id: :obj:`str`
    :param transfer_star_count: Optional. Number of Telegram Stars that must be paid to transfer the gift; omitted if the bot cannot transfer the gift
    :type transfer_star_count: :obj:`int`
    """

    def __init__(
        self,
        gift: "UniqueGift" = None,
        origin: "str" = None,
        last_resale_star_count: int = None,
        owned_gift_id: "str" = None,
        transfer_star_count: "int" = None,
        next_transfer_date: int = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.gift = gift
        self.origin = origin
        self.last_resale_star_count = last_resale_star_count
        self.owned_gift_id = owned_gift_id
        self.transfer_star_count = transfer_star_count
        self.next_transfer_date = next_transfer_date

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["UniqueGiftInfo"]:
        return (
            UniqueGiftInfo(
                gift=UniqueGift._parse(me, d.get("gift")),
                origin=d.get("origin"),
                last_resale_star_count=d.get("last_resale_star_count"),
                owned_gift_id=d.get("owned_gift_id"),
                transfer_star_count=d.get("transfer_star_count"),
                next_transfer_date=d.get("next_transfer_date"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import Union


class SetChatAdministratorCustomTitle:
    async def set_chat_administrator_custom_title(
        self: "tgram.TgBot", chat_id: Union[int, str], user_id: int, custom_title: str
    ) -> bool:
        """
        Use this method to set a custom title for an administrator in a supergroup promoted by the bot.
        Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#setchatadministratorcustomtitle

        :param chat_id: Unique identifier for the target chat or username of the target supergroup
            (in the format @supergroupusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param user_id: Unique identifier of the target user
        :type user_id: :obj:`int`

        :param custom_title: New custom title for the administrator;
            0-16 characters, emoji are not allowed
        :type custom_title: :obj:`str`

        :return: True on success.
        :rtype: :obj:`bool`
        """

        result = await self(
            "setChatAdministratorCustomTitle",
            chat_id=chat_id,
            user_id=user_id,
            custom_title=custom_title,
        )
        return result["result"]



import tgram
from typing import Union


class SetChatDescription:
    async def set_chat_description(
        self: "tgram.TgBot", chat_id: Union[int, str], description: str = None
    ) -> bool:
        """
        Use this method to change the description of a supergroup or a channel.
        The bot must be an administrator in the chat for this to work and must have the appropriate admin rights.

        Telegram documentation: https://core.telegram.org/bots/api#setchatdescription

        :param chat_id: Unique identifier for the target chat or username of the target channel
            (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param description: Str: New chat description, 0-255 characters
        :type description: :obj:`str`

        :return: True on success.
        :rtype: :obj:`bool`
        """

        result = await self(
            "setChatDescription",
            chat_id=chat_id,
            description=description,
        )
        return result["result"]



import tgram

from typing import List, Optional


class InlineQueryB:
    async def answer(
        self: "tgram.types.InlineQuery",
        results: List["tgram.types.InlineQueryResult"],
        cache_time: int = None,
        is_personal: bool = None,
        next_offset: str = None,
        button: "tgram.types.InlineQueryResultsButton" = None,
    ) -> bool:
        """
        Sends answers to an inline query.

        Parameters:
        - results (List[tgram.types.InlineQueryResult]): A list of results for the inline query.
        - cache_time (int, optional): The maximum amount of time in seconds that the result of the inline query may be cached on the server.
        - is_personal (bool, optional): Pass True if results may be cached on the server side only for the user that sent the query. By default, results may be returned to any user who sends the same query.
        - next_offset (str, optional): Pass the offset that a client should send in the next query with the same text to receive more results. Pass an empty string if there are no more results or if you don’t support pagination. Offset length can’t exceed 64 bytes.
        - button (tgram.types.InlineQueryResultsButton, optional): A button to be shown above inline query results.

        Returns:
        - bool: True on success.
        """
        return await self._me.answer_inline_query(
            inline_query_id=self.id,
            results=results,
            cache_time=cache_time,
            is_personal=is_personal,
            next_offset=next_offset,
            button=button,
        )

    @property
    def user(self: "tgram.types.InlineQuery") -> Optional["tgram.types.User"]:
        """
        Returns the user who sent the inline query.

        Returns:
        - Optional[tgram.types.User]: The user who sent the inline query.
        """
        return self.from_user

    sender_user = user



import tgram
from typing import List
from tgram.types import Sticker


class GetCustomEmojiStickers:
    async def get_custom_emoji_stickers(
        self: "tgram.TgBot", custom_emoji_ids: List[str]
    ) -> List[Sticker]:
        """
        Use this method to get information about custom emoji stickers by their identifiers.
        Returns an Array of Sticker objects.

        :param custom_emoji_ids: List of custom emoji identifiers. At most 200 custom emoji identifiers can be specified.
        :type custom_emoji_ids: :obj:`list` of :obj:`str`

        :return: Returns an Array of Sticker objects.
        :rtype: :obj:`list` of :class:`tgram.types.Sticker`
        """

        result = await self(
            "getCustomEmojiStickers",
            custom_emoji_ids=custom_emoji_ids,
        )
        return [Sticker._parse(me=self, d=i) for i in result["result"]]



import tgram
from typing import Union


class DemoteChatMember:
    async def demote_chat_member(
        self: "tgram.TgBot", chat_id: Union[int, str], user_id: int
    ) -> bool:
        """
        Use this method to demote a user in a supergroup or a channel. The bot must be an administrator

        Telegram documentation: null

        :param chat_id: Unique identifier for the target chat or username of the target channel (
            in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param user_id: Unique identifier of the target user
        :type user_id: :obj:`int`

        :return: True on success.
        :rtype: :obj:`bool`
        """

        result = await self(
            "promoteChatMember",
            chat_id=chat_id,
            user_id=user_id,
            is_anonymous=False,
            can_manage_chat=False,
            can_delete_messages=False,
            can_manage_video_chats=False,
            can_restrict_members=False,
            can_promote_members=False,
            can_change_info=False,
            can_invite_users=False,
            can_post_stories=False,
            can_edit_stories=False,
            can_delete_stories=False,
            can_post_messages=False,
            can_edit_messages=False,
            can_pin_messages=False,
            can_manage_topics=False,
        )
        return result["result"]
